#ifndef EKK_C_API
#define EKK_C_API
#include <stdio.h>
/* ekk_c_api.h   -  OSL header file                           */
/* IBM Optimization Solutions Library                         */
/* Restricted Materials of IBM                                */
/* Copyright: International Business Machines Corp, 1998-2000 */

#ifndef OSLLIBAPI

#if defined (OSLMSDLL)
#   define OSLLIBAPI __declspec(dllexport)
#   define OSLLINKAGE  __stdcall
#   define OSLLINKAGE_CB  __cdecl
#elif defined (OSLMSSTAT)
#   define OSLLINKAGE  __stdcall
#   define OSLLIBAPI
#   define OSLLINKAGE_CB  __cdecl
#elif defined (EKKC_OE)
#   define OSLLINKAGE  
#   define OSLLIBAPI
#   ifdef __cplusplus
#     define OSLLINKAGE_CB  __cdecl
#   else
#     define OSLLINKAGE_CB  
#   endif
#else
#   define OSLLIBAPI
#   define OSLLINKAGE
#   define OSLLINKAGE_CB 
#endif

#endif


#define OSL_INFINITY 1.0e31

/* minimum needed for user */
typedef struct EKKModel EKKModel;
typedef struct EKKContext EKKContext;

/* for each set entry*/
typedef struct{
  int sequence; /* 0 is first variable */
  unsigned short number_up; /* number of times branched up on variable */
  unsigned short number_down; /* number of times branched down on variable */
  double down_pseudo_cost;
  double up_pseudo_cost; /* could also be reference row entry */
} EKKSetEntry;

/* for each int set */
typedef struct{
  int type; /* 4 is single, 1 is IBM SOS 1 , 2 is SOS 1 , 3 is SOS 2*/
  int priority;
  int number_in_set;
  EKKSetEntry * entries_in_set;
} EKKIntegerSet ;

struct EKKVector {
  int *index;
  double *element;
  int numNonZero;
};
typedef struct EKKVector EKKVector;

struct EKKMatrixCopy {
  int *index;
  int *start;
  double *element;
};
typedef struct EKKMatrixCopy EKKMatrixCopy;


#ifdef __cplusplus
extern "C"{
#endif

/****************************************************************/
/*     These are the most important functions                   */
/*     note that non-zero return codes indicate some error      */
/*     The notes here are an abbreviated version of those in    */
/*     the (mythical) OSL primer                                */ 
/*     (this file is dedicated to David L. Jensen so that he    */
/*     he does not have to read any real documentation)         */
/*     ** If you think something needs moving/clarifying then   */
/*     inform John Forrest                                      */
/****************************************************************/

  /*****
  1) a context for OSL has to be set up.  This is done just once in a
     program.  The context pointer is passed whenever a new model is
     created.  It is also passed to close down the context */

  /* sets up entire OSL context and returns the context pointer */
OSLLIBAPI EKKContext *  OSLLINKAGE ekk_initializeContext();

  /* deletes entire OSL context.  All existing models will be deleted
     and any other areas OSL has created - including the context itself
     so there is no need to free "context" */
OSLLIBAPI void OSLLINKAGE ekk_endContext(EKKContext * context);

  /*****
  2) within the OSL context there can be as many models as the user needs.
     For each new model the OSL context is passed and an optional name */

  /* return a new empty model - zero rows and zero columns */
OSLLIBAPI EKKModel *  OSLLINKAGE ekk_newModel(EKKContext * env,
					      const char * name);
  /* delete a model - all memory used is freed */
OSLLIBAPI int OSLLINKAGE ekk_deleteModel(EKKModel * model);

  /*****
  3) One of the simplest ways to obtain a real model is from an MPS
     format file.  If you know that the file is true MPS format i.e.
     fixed fields and eight character names then use ekk_importModel.
     If the model has a blank name, and there is a valid name on the
     MPS file then this will become the name of the model.  For more
     ways to build a model see section 6) */
  /* imports a model in MPS format */
OSLLIBAPI int OSLLINKAGE ekk_importModel(EKKModel * model , 
					 const char * fileName);
  /* If the file is in free format or has longer names (up to 24
     characters) then use ekk_importModelFree which is similar.
     When the model has been imported then the names will be compressed. */
OSLLIBAPI int OSLLINKAGE ekk_importModelFree(EKKModel * model , 
					    const char * fileName);
  /*****
  4) There are several algorithms available to solve the model.  Probably
     the most useful is the simplex algorithm.  We have developed a
     packaged version which automatically does pre and post processing.
     This takes one of several modes  - 0=choose primal or dual, 1=primal,
     2=dual and 3= primal after an experimental "crash".  
     We would suggest 1 for a novice user  */

OSLLIBAPI int OSLLINKAGE ekk_simplex(EKKModel * model,int mode);

  /*****
  5) After solving the model we need to get the results.  One simple way
     is just to print the solution.  We will see later how to vary the
     amount printed */

OSLLIBAPI int OSLLINKAGE ekk_printSolution(EKKModel * model);

  /************************************************************************/
  /************************************************************************

	That is enough to get useful work done, although we may be 
	able to solve large problems faster by changing parameters 
	as indicated in later sections.

	So here is a valid OSL fragment 
        {
	  EKKContext * context = ekk_initializeContext();
	  EKKModel * model = ekk_newModel(context," ");
	  ekk_importModel(model,"25fv47.mps");
	  ekk_simplex(model,1);
	  ekk_printSolution(model);
	  ekk_endContext(context);
        }

    The contents of the next sections are:

    6) Other ways to build a model.  
    7) Some other algorithms and ways to tune for them.
    8) Message handling, printing and accessing data
    9) Mixed Integer Problems

       That may be enough for your first applications.  When you feel
       confident then look at sections 10 through 15 (and in the primer) 
       for more advanced features.  At the end there are prototypes for
       features which are only in the reference guide.    */

  /************************************************************************/
  /************************************************************************

  6) It is not necessarily convenient to use the MPS format, so
     here are some ways to build up a model. (Note that for matrix
     coefficients the row and column indices conform to C practice i.e.
     the first row or column is 0 */

  /* functions such as ekk_loadRimModel copy in data (the const gives a clue).
     If NULL is passed then lower bounds on the row activities are set to
     - infinity (-OSL_INFINITY) and upper bounds to + infinity.  For the
     columns the objective coefficients would be zero, as would the lower
     bounds on the column activities while the upper bound would be infinity.

     There are at least three ways to use ekk_loadRimModel :
     a) Load the entire "Rim" i.e. for a ten row problem with
        fifteen columns, you would pass in all arrays.  Then use
	ekk_addColumnElementBlock or ekk_addElementBlock to fill in the
	matrix coefficients.
     b) A more natural way might be to create it by row.  In this case
        numberRows would be zero, while numberColumns would be fifteen.
	Then ten calls to ekk_addOneRow would be made to add in the rows.
     c) Or of course you could create it by column, so numberRows would
        be ten and numberColumns zero and lowerRow and upperRow would
        be passed.  Then ekk_addOneColumn would be called fifteen times.
     In fact you have total flexibility as to how you build the model
     and you may use different methods depending on whether you want
     simplicity, speed or compact memory. */

  /* This defines the current model size as numberRows x numberColumns and 
     fills in arrays in the model using defaults or the array values. After
     this the size can be changed by adding rows, columns or both.  A 
     second call to ekk_loadRimModel deletes all data before copying
     new arrays.  For alternative ways to create a model see ekk_resizeModel */

OSLLIBAPI int OSLLINKAGE ekk_loadRimModel(EKKModel * model , int numberRows ,
    const double * lowerRow , const double * upperRow ,
    int numberColumns , const double * objective , const double * lowerColumn ,
    const double * upperColumn);

  /* This adds one row to the problem giving lower and upper bounds
     on row activities plus the nonzero elements in row */
OSLLIBAPI int OSLLINKAGE ekk_addOneRow(EKKModel * model , double lowerBound ,
                   double upperBound , int numberElements,
                   const int * indexColumn , const double * element);
  /* while this adds a column and the nonzero elements in column */
OSLLIBAPI int OSLLINKAGE ekk_addOneColumn(EKKModel * model , 
                   double objectiveCoefficient, double lowerBound ,
                   double upperBound , int numberElements,
                   const int * indexRow , const double * element);
  /* Matrices can be stored as triplets of row index, column index and
     value.  While this is flexible, algorithmic performance may suffer.
     see ekk_mergeBlocks in section 7) for tips.  A more efficient way
     is to store the matrix column ordered where the row indices and 
     coefficients are stored in column order with the start of each column
     being given - so if the starts are 0,5,7,15.. then the first few
     columns have 5, 2, 8 nonzero coefficients.  The two functions below
     return nonzero on failure and zero on success (the same is true for other
     addElement functions) */
  /* this adds as triplets */
OSLLIBAPI int OSLLINKAGE ekk_addElementBlock(EKKModel * model ,
    int numberElements ,
    const int * indexRow , const int * indexColumn , const double * element);
  /* while this adds as column ordered */
OSLLIBAPI int OSLLINKAGE ekk_addColumnElementBlock(EKKModel * model , 
    int numberColumns,
    const int * indexRow , const int * startOfEachColumn , 
						   const double * element);
  /************************************************************************/
  /************************************************************************

  7) Algorithms and how to tune.
     The most basic option is to decide on the direction of optimization.
     The default for OSL is to minimize.  To change this use ekk_setRmaxmin
     (or the equivalent ekk_setMaximize or ekk_setMinimize).
     If you have used previous versions of OSL the name rmaxmin may seem
     familiar.  We have kept the same names so that the old documentation
     is still useful.  In previous versions you would have got the
     values of all Real scalars by ekkrget(rtcode,dspace,oslr,oslrln) - 
     changed one or two and then called ekkrset.  This has been made 
     more lightweight and there is no need to get the value of a scalar
     to modify it.  So ekk_setRmaxmin(model,-1.0) sets the direction
     to maximization (1.0 is minimize and 0.0 says ignore the objective).
     ekk_setMaximize(model) is equivalent to ekk_setRmaxmin(model,-1.0).
     ekk_set and ekk_get come in pairs and in this version we allow you
     to set some that were not possible before - see ekk_setRobjvalue
     for why you might want to do that.  */

  /***** note that all get/set functions come in pairs which all have the
     same format *****/
  /* get current value of rmaxmin */
OSLLIBAPI double OSLLINKAGE ekk_getRmaxmin(EKKModel * model);
  /* set a new value of rmaxmin */
OSLLIBAPI int OSLLINKAGE ekk_setRmaxmin(EKKModel * model , double value);
  /* you may prefer this way */
OSLLIBAPI int OSLLINKAGE ekk_setMinimize(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setMaximize(EKKModel * model);

  /*****
	General ways to tune.
	One very effective way to solve a large problem faster is to make
	it smaller!  Modeling languages will generate a constraint even
	if it of the form SUM over i of x(i)  <= 0.  Obviously all the
	x(i) must take the value zero.  Similarly a constraint like
        2.5 x <= 10.0 can be replaced by putting an upper bound of 4.0
	on x.  There are many more things that can be done, see the 
	primer for more details.  In OSL this is done by ekk_preSolve, but
	after solving ekk_postSolve should be called to expand back the
	solution.  In most cases this solution is optimal, but there
	may be cases where the value of the objective function is optimal
	but there are some minor dual infeasibilities so it is a good
	idea to call ekk_primalSimplex again. 
        ekk_preSolve returns a value of 1 when it determines that the
        problem in infeasible. */
  /* if fileName NULL then save to memory otherwise save to file */
  /* mode=3 is a good setting, 0,1 or 2 are less aggressive */
OSLLIBAPI int OSLLINKAGE ekk_preSolve(EKKModel * model , int mode , 
				      const char * fileName);
OSLLIBAPI int OSLLINKAGE ekk_postSolve(EKKModel * model , 
				       const char * fileName);

  /*****
	In some problems all coefficients are 1, while in others they
	may vary from 1.0e-8 to 1.0e10.  Obviously the latter ones
	are difficult to solve but it will considerably improve the
	performance to scale the problem to reduce this variance. 
	There is less need for scaling with a barrier algorithm */
OSLLIBAPI int OSLLINKAGE ekk_scale(EKKModel * model);

  /*****
	When the model is created an implicit "do-nothing" basis is
	created.  In most cases it is a good idea do a "crash" 
	which puts variables into the basis.  Mode 2 is good for
	a both simplex methods, while 1 may be better for primal
	simplex.  It has no effect on the barrier algorithm. */
OSLLIBAPI int OSLLINKAGE ekk_crash(EKKModel * model, int mode);

  /*****
	PreSolve or scale all convert the matrix into a column
	ordered version.  This is the most efficient storage
	structure for the algorithms.  ekk_mergeBlocks merges all
	blocks of the matrix into one.  If ifColumnOrdered is 1 then
	this will be column ordered, otherwsie triplets. */
OSLLIBAPI int OSLLINKAGE ekk_mergeBlocks(EKKModel * model , 
					 int ifColumnOrdered);

  /*****
	After ekk_presolve, the problem may be much smaller but while
	the number of matrix coefficients will have been reduced the
	fixed variables will still be in the problem.  This makes 
	very little difference if there not very many, but if the
	problem size has been dramatically reduced then this slight
	overhead may not be so slight.  If compression is on then
	the model is scrunched down as much as possible.  This improves
	the speed, the only drawback being for more complex applications
	which need to access the variables during solution.  So nearly
	always set compression on. */
OSLLIBAPI void OSLLINKAGE ekk_setCompressOn(EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_setCompressOff(EKKModel * model);

  /*****
	ekk_simplex tries to help the user by automatically doing a
	presolve and scale.  However advanced users may wish to switch 
	these options off in ekk_simplex.  Defaults are on. */
OSLLIBAPI void OSLLINKAGE ekk_setPresolveOn(EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_setPresolveOff(EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_setScaleOn(EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_setScaleOff(EKKModel * model);

  /*****
	The final tuning for speed option is whether to have a copy
	of the matrix stored by row.  In recent years some very large
	models have been developed and new implementations of the
	simplex algorithms take advantage of some of their properties.
	Having a copy of the matrix stored by row can make some large
	models go several times faster when using the simplex method.
	Of course this does use more memory so there may be a trade off.
	Setting the useRowCopy value to 1 switches on, 0 switches off. */
OSLLIBAPI int OSLLINKAGE ekk_setIuseRowCopy(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_getIuseRowCopy(EKKModel * model);

  /****
       The two general simplex methods are the primal and dual methods.
       For the primal simplex algorithm there is one option - the startup mode.
       1 is reccommended for initial solution, while 3 is good for continuing
       on from a previous solution or an imported basis.

       The calling sequence for the dual method is even simpler and there
       is also an "either" method which guesses which will be better.  For
       production runs the user should see which works better 

       The ekk_simplex introduced earlier is very approximately (mode==1):

        user_primalSimplex(EKKModel * model) 
        {
	  ekk_preSolve(model,3,"/tmp/save.fil");
	  ekk_scale(model);
	  ekk_setIuseRowCopy(model,1);
	  ekk_crash(model,1);
	  ekk_primalSimplex(model,1);
	  ekk_postSolve(model,"/tmp/save.fil");
	  ekk_primalSimplex(model,3);
	  }                                         */

OSLLIBAPI int OSLLINKAGE ekk_primalSimplex(EKKModel * model , int startUpMode);
OSLLIBAPI int OSLLINKAGE ekk_dualSimplex(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_eitherSimplex(EKKModel * model);

  /*****
	There are two "interior point" or "barrier" methods in OSL.
	The older one implements a Primal Dual Predictor Corrector
	algorithm.  The second regularizes the model and is more
	robust but is solving a perturbed model.  In general ekk_barrier
	is recommended.  You will probably want a basic solution
	as from the simplex methods so ekk_crossover should be called
	afterwards.  This normally is fast, but it can help to ask the
	barrier algorithm to obtain a more accurate solution.  This is
	done by setting Rpdgaptol to be smaller (e.g. 1.0e-12) than
	the default of 1.0e-7.  This tolerance is the desired relative gap
	between the primal and dual solutions. Mode in ekk_barrier 
	should normally be 0. */

OSLLIBAPI int OSLLINKAGE ekk_barrier(EKKModel * model , int mode);
OSLLIBAPI int OSLLINKAGE ekk_crossover(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRpdgaptol(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setRpdgaptol(EKKModel * model , double value);

  /***** For the regularized interior point method - see Saunders and
	 Tomlin ??.  A perturbation of delta is applied on the dual side
	 and gamma on the primal side.  If this is too small there may
	 be a fatal loss of rank.  If too big then the answer may be
	 slightly inaccurate.  If you are going to do crossover then
	 for a first try leave at default safe values (1.0e-3).  To
	 use less perturbation set smaller e.g. 1.0e-6.  The densecol
	 parameter should not be needed, but can be set to take columns
	 with at least that many elements out of the cholesky factorization
	 used in interior point methods.  This can make a substantial
	 difference to speed and memory requirements. Again mode should be 0.*/

OSLLIBAPI int OSLLINKAGE ekk_regularized(EKKModel * model , int mode);
OSLLIBAPI double OSLLINKAGE ekk_getRregularizedGamma(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRregularizedDelta(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setRregularizedGamma(EKKModel * model ,
						  double value);
OSLLIBAPI int OSLLINKAGE ekk_setRregularizedDelta(EKKModel * model , 
						  double value);
OSLLIBAPI int OSLLINKAGE ekk_getIdensecol(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setIdensecol(EKKModel * model , int value);

  /************************************************************************/
  /************************************************************************

  8) Message handling, printing and accessing data.
     For small problems it is very easy to be overwhelmed by the
     amount of messages, so once you understand what is happening it
     is a good idea to switch off some informational messages.  This
     can be done one at a time or several at a time. */
  /* Switch on or off message printing - one at a time */
OSLLIBAPI int OSLLINKAGE ekk_messagePrintOn(EKKModel * model ,
					    int messageNumber);
OSLLIBAPI int OSLLINKAGE ekk_messagePrintOff(EKKModel * model ,
					     int messageNumber);
  /* Switch on or off message printing - a range at a time */
OSLLIBAPI int OSLLINKAGE ekk_messagesPrintOn(EKKModel * model ,
			int startMessageNumber,int endMessageNumber);
OSLLIBAPI int OSLLINKAGE ekk_messagesPrintOff(EKKModel * model ,
                        int startMessageNumber,int endMessageNumber);
  /* of course you may want even more information.  The third parameter
     is not used at present but if ifPrintParameters >= 0 then information
     is printed on many parameters passed into OSL.  If >0 then all arrays
     <= the value will be printed */
OSLLIBAPI void OSLLINKAGE ekk_setDebug(EKKModel * model,
				       int ifPrintParameters,int verifyLevel);

  /*****
	To change the what printed by ekk_printSolution - use linelen to
	change the length of each line, solmask to print which variables
	have information printed - 0,15 or 16 print all,
	1 - rows, 2 - colums, 4 - nonzero activities, 8 - infeasible
	activities.  These are ORed so to print just nonzero columns
	set it to 6.  prtinfomask controls what is printed for each
	variable.  This also is ORed so the default of 31 prints
	a preamble plus names, status and activity for each variable.
	To add duals it would be 63 - see reference for full details. 

	With all this output, you may wish to put it out on another file.
	To do this change the logfile with LogfileName ("stdout" is
	recognized) or to use the same files as the rest of your program
	use LogfileFilePointer.   */
OSLLIBAPI int OSLLINKAGE ekk_getIlinelen(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIprtinfomask(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIsolmask(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setIlinelen(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIprtinfomask(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIsolmask(EKKModel * model , int value);
OSLLIBAPI const char * OSLLINKAGE ekk_getLogfileName(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setLogfileName(EKKModel * model ,
					    const char * value);
OSLLIBAPI FILE * OSLLINKAGE ekk_getLogfileFilePointer(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setLogfileFilePointer(EKKModel * model ,
           FILE * logfp);

  /*****
	Your program may also wish to test other values.  Here are some -
	numpinf  - number of primal infeasibilities
	numdinf  - number of dual infeasibilities
        iternum  - total number of iterations
        probstat - status -1 = unknown, 0 = optimal, 1 = primal infeasible
	                   2 = dual infeasible (unbounded), 3 = stopped
	objvalue - value of objective function
	sumpinf  - sum of primal infeasibilities
	sumdinf  - sum of dual infeasibilities
        -- you can also set these.  This may seem odd but in an
	iterative algorithm it can make for cleaner coding to set
	the model to be infeasible or have a large objective value
	before starting.                                   */
OSLLIBAPI int OSLLINKAGE ekk_getInumpinf(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getInumdinf(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIiternum(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIprobstat(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRobjvalue(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRsumpinf(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRsumdinf(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setInumpinf(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setInumdinf(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIiternum(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIprobstat(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setRobjvalue(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRsumpinf(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRsumdinf(EKKModel * model , double value);

  /*****
	Also most of the data inside the model is accessible by use of
	pointers.  While this is mainly for reading data, it is also legal
	to modify data using these pointers although users should take
	great care.  Functions such as ekk_replaceElements or 
	ekk_copyRowLower are safer but in some cases users may need the
	flexibility of direct modification.  Of course to access this
	data the number of rows or columns in the model must be known and
	this information is available using ekk_getInumrows/ekk_getInumcols.
	There are "set" versions of these whose effect is to resize (only
	increasing) the model.  */

OSLLIBAPI int OSLLINKAGE ekk_getInumrows(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getInumcols(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setInumrows(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setInumcols(EKKModel * model , int value);

OSLLIBAPI const double * OSLLINKAGE ekk_rowlower(EKKModel * model);
OSLLIBAPI const double * OSLLINKAGE ekk_rowacts(EKKModel * model);
OSLLIBAPI const double * OSLLINKAGE ekk_rowupper(EKKModel * model);
OSLLIBAPI const double * OSLLINKAGE ekk_rowduals(EKKModel * model);
OSLLIBAPI const int    * OSLLINKAGE ekk_rowstat(EKKModel * model); 
OSLLIBAPI const double * OSLLINKAGE ekk_collower(EKKModel * model);
OSLLIBAPI const double * OSLLINKAGE ekk_colsol(EKKModel * model); 
OSLLIBAPI const double * OSLLINKAGE ekk_colupper(EKKModel * model);
OSLLIBAPI const double * OSLLINKAGE ekk_colrcosts(EKKModel * model);
OSLLIBAPI const int    * OSLLINKAGE ekk_colstat(EKKModel * model);
OSLLIBAPI const double * OSLLINKAGE ekk_objective(EKKModel * model);

/* These allocate a buffer that the user can modify;
 * the buffer must either be later ekk_free'd
 */
OSLLIBAPI double * OSLLINKAGE ekk_getRowlower(EKKModel * model);
OSLLIBAPI double * OSLLINKAGE ekk_getRowacts(EKKModel * model);
OSLLIBAPI double * OSLLINKAGE ekk_getRowupper(EKKModel * model);
OSLLIBAPI double * OSLLINKAGE ekk_getRowduals(EKKModel * model);
OSLLIBAPI int    * OSLLINKAGE ekk_getRowstat(EKKModel * model); 
OSLLIBAPI double * OSLLINKAGE ekk_getCollower(EKKModel * model);
OSLLIBAPI double * OSLLINKAGE ekk_getColsol(EKKModel * model); 
OSLLIBAPI double * OSLLINKAGE ekk_getColupper(EKKModel * model);
OSLLIBAPI double * OSLLINKAGE ekk_getColrcosts(EKKModel * model);
OSLLIBAPI int    * OSLLINKAGE ekk_getColstat(EKKModel * model);
OSLLIBAPI double * OSLLINKAGE ekk_getObjective(EKKModel * model);

/* These accept buffers from the corresponding "get" routines;
 * they do NOT free the buffer - use ekk_free
 */
OSLLIBAPI int OSLLINKAGE ekk_setRowlower(EKKModel * model,
					 const double * fromArray);
OSLLIBAPI int OSLLINKAGE ekk_setRowacts(EKKModel * model,
					const double * fromArray);
OSLLIBAPI int OSLLINKAGE ekk_setRowupper(EKKModel * model,
					 const double * fromArray);
OSLLIBAPI int OSLLINKAGE ekk_setRowduals(EKKModel * model,
					 const double * fromArray);
OSLLIBAPI int OSLLINKAGE ekk_setRowstat(EKKModel * model,
					const int * fromArray);
OSLLIBAPI int OSLLINKAGE ekk_setCollower(EKKModel * model,
					 const double * fromArray);
OSLLIBAPI int OSLLINKAGE ekk_setColsol(EKKModel * model,
				       const double * fromArray);
OSLLIBAPI int OSLLINKAGE ekk_setColupper(EKKModel * model,
					 const double * fromArray);
OSLLIBAPI int OSLLINKAGE ekk_setColrcosts(EKKModel * model,
					  const double * fromArray);
OSLLIBAPI int OSLLINKAGE ekk_setColstat(EKKModel * model,
					const int * fromArray);
OSLLIBAPI int OSLLINKAGE ekk_setObjective(EKKModel * model,
					  const double * fromArray);



OSLLIBAPI const char * const * OSLLINKAGE 
                                     ekk_rowNamePointers(EKKModel * model);
OSLLIBAPI const char * const * OSLLINKAGE 
                                     ekk_columnNamePointers(EKKModel * model);

/* note: the auxiliary region is created when problem is infeasible
 * or unbounded and gives extra information.
 */
OSLLIBAPI const double * OSLLINKAGE ekk_rowaux(EKKModel * model);
OSLLIBAPI const double * OSLLINKAGE ekk_colaux(EKKModel * model);

  /************************************************************************/
  /************************************************************************

  9) Mixed Integer problems.  The extra information needed for solving
     integer problems can be on the MPS file or it can be added for
     a model which has been imported or loaded from arrays.  The simplest
     way is to mark the integer variables using ekk_markAsInteger, but
     to allow more control it is better to add varaibles in sets.  Each
     set may have a different priority (1 highest).  Also pseudo costs
     may be defined to aid the search.  The definition of SOS (Special
     Ordered Sets) is - type 1 => only one variable in set may be non
     zero and the order is useful, type 2 => only two variables may be
     non zero (if two then they must be "adjacent" in terms of the
     ordering of the reference row) while type 3 => an IBM misinterpretation
     which says exactly one variable must be 1.0 */

OSLLIBAPI int OSLLINKAGE ekk_markAsInteger(EKKModel * model , int iColumn);
OSLLIBAPI int OSLLINKAGE ekk_addIntegerSet(EKKModel * model , int priority ,
       int number , const int * columns ,
       const double * downPseudoCost , const double * upPseudoCost);
OSLLIBAPI int OSLLINKAGE ekk_addSOS1Set(EKKModel * model , int priority ,
       int number , const int * columns ,
       const double * downPseudoCost , const double * referenceRow);
OSLLIBAPI int OSLLINKAGE ekk_addSOS2Set(EKKModel * model , int priority ,
       int number , const int * columns ,
       const double * downPseudoCost , const double * referenceRow);
OSLLIBAPI int OSLLINKAGE ekk_addSOS3Set(EKKModel * model , int priority ,
       int number , const int * columns ,
       const double * downPseudoCost , const double * referenceRow);

  /*****
	Simple branch and bound is done by calling ekk_branchAndBound with
	optional files for saving bound modifications and basis information.
	For small problems, here is one place where you may wish to switch
	most messages as that will improve performance e.g.

	    ekk_messagePrintOff(model,100);
	    ekk_messagePrintOff(model,95);
	    ekk_messagePrintOff(model,87);
	    ekk_messagePrintOff(model,6);
	    ekk_messagePrintOff(model,102);
	    ekk_messagePrintOff(model,34);
  */

OSLLIBAPI int OSLLINKAGE ekk_branchAndBound(EKKModel * model , 
                         const char * matrixFile ,
		             	 const char * basisFile);

  /***** The most useful parameter to be set is bbcutoff which gives
	 an upper bound for search i.e. all nodes with objective value
	 greater than this will be pruned.  In Branch and Bound all
	 such parameters are in terms of minimization */

OSLLIBAPI int OSLLINKAGE ekk_getImaxnodes(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getImaxsols(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRbbcutoff(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setImaxnodes(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setImaxsols(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setRbbcutoff(EKKModel * model , double value);


/****************************************************************/
/*     These are also commonly important functions              */
/*     for more information see OSL primer                      */
/****************************************************************/


  /*****
	Other algorithms include quadratic simplex and barrier.  To use
	these the quadratic part of the model has to be added.  
	ekk_importQuadraticData reads from a file in MPS like format.
        For loadQuadraticData, if type is 1 then the data is passed as 
	triplets, while for 2 it is passed column ordered.  It is 
	preferable not to use scaling or preSolve with quadratic.
        mode in quadratic simplex is 0 to continue from a previous
	quadratic solution, 1 to solve from scratch.  For the barrier
	method a variant of the regularized method is used and the
	same notes on parameters apply - again 0 is recommended
	for mode for ekk_quadraticBarrier.  Finally ekk_quadraticParametrics
	solves the parametric quadratic problem. */

OSLLIBAPI int OSLLINKAGE ekk_importQuadraticData(EKKModel * model ,
                     const char * fileName);
OSLLIBAPI int OSLLINKAGE ekk_loadQuadraticData(EKKModel * model ,
            int type , int  numberElements, const int * row, 
            const int * column, const double * element);
#ifndef EKK_THREADSAFE
OSLLIBAPI int OSLLINKAGE ekk_quadraticSimplex(EKKModel * model, int mode);
OSLLIBAPI int OSLLINKAGE ekk_quadraticParametrics(EKKModel * model,
     const double * parrhs , const double * parobj , 
     double lambdaMin, double lambdaMax);
#endif
OSLLIBAPI int OSLLINKAGE ekk_quadraticBarrier(EKKModel * model, int mode);

  /*****
	If the model is a network, then a network code can be used. Try
	startup 1 and then if you need more information look at reference */

OSLLIBAPI int OSLLINKAGE ekk_primalNetwork(EKKModel * model , int startup);
OSLLIBAPI int OSLLINKAGE ekk_dualNetwork(EKKModel * model , int startup);

  /*****
	Import/export etc.  Import reads data in MPS format (if eight
	character names), while export writes data in MPS format.  This
	is good for portability.  If you wish to change the length of names
	then numchar should be set.  If you created the model without names
	then export will create default names.  You can create your own
	names (for whatever use) by using one of the functions below; you
	can also delete all names to save space by ekk_deleteNames.
	ekk_saveModel, ekk_restoreModel save and restore all or part of 
	model in an internal format - which parts are affected by 
	ekk_setImodelmask - see reference.  */
OSLLIBAPI int OSLLINKAGE ekk_exportModel(EKKModel * model, 
					 const char * fileName ,
					 int ifCharacter ,  int numberFields);
OSLLIBAPI int OSLLINKAGE ekk_exportBasis(EKKModel * model, 
					 const char * fileName ,
					 int ifCharacter);
OSLLIBAPI int OSLLINKAGE ekk_importBasis(EKKModel * model, 
					 const char * fileName);
OSLLIBAPI int OSLLINKAGE ekk_saveModel(EKKModel * model ,  
				       const char * fileName);
OSLLIBAPI int OSLLINKAGE ekk_restoreModel(EKKModel * model ,  
					  const char * fileName);
OSLLIBAPI int OSLLINKAGE ekk_getInumchar(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setInumchar(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_getImodelmask(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setImodelmask(EKKModel * model , int value);
/* createNames just creates Rnnnnnnn and Cnnnnnnn names and sets name
   length  - use copy(Row,Column)names to copy in names */
OSLLIBAPI int OSLLINKAGE ekk_createNames(EKKModel * model, 
					 int lengthOfEachName);
/* names are copied from fromArray into indicated part of model names.
     If names do not exist they will be created using ekk_createNames.
     names will be padded or truncated.  Each entry is presumed to be
     at most lengthOfInName (as in ekk_setInumchar) */
OSLLIBAPI int OSLLINKAGE ekk_copyRowNames(EKKModel * model,
                const char * const * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_copyColumnNames(EKKModel * model,
                const char * const * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_deleteNames(EKKModel * model);

  /*****
	Other ways of replacing data.  The less safe way is to get a
	pointer to internal data and use that.  A safer way to replace part
	of an array is to use on of the ekk_copy functions (as for names
	above). The fromArray is user data which will be copied into
	the indicated part of model data (from toStart to toEnd-1). */

OSLLIBAPI int OSLLINKAGE ekk_copyRowlower(EKKModel * model,
                const double * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_copyRowacts(EKKModel * model,
                const double * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_copyRowupper(EKKModel * model,
                const double * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_copyRowduals(EKKModel * model,
                const double * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_copyRowstat(EKKModel * model,
                const int * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_copyCollower(EKKModel * model,
                const double * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_copyColsol(EKKModel * model,
                const double * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_copyColupper(EKKModel * model,
                const double * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_copyColrcosts(EKKModel * model,
                const double * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_copyColstat(EKKModel * model,
                const int * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_copyObjective(EKKModel * model,
                const double * fromArray, int toStart, int toEnd);

  /*****
	For matrix coefficients there are several ways to add and
	modify data.  First (normally this should be used for LOOKING
	at data) pointers can be retrieved for each block of the model
	(normally one - column ordered) - the number and type are
	returned by ekk_getInumblocks (no set) and ekk_blockType. 
	To get a copy of the matrix by row or column which belongs
	to user use ekk_createUserMatrix.  This can be deleted by
	ekk_deleteUserMatrix and examined using ekk_rowcc etc.  
	cc is for column copy, rc for row copy so ekk_rowrc gives
	row starts for row copy, while ekk_rowcc gives row indices for
	column copy. */
OSLLIBAPI const int * OSLLINKAGE ekk_blockRow(EKKModel * model, 
					int whichBlock);
OSLLIBAPI const int * OSLLINKAGE ekk_blockColumn(EKKModel * model, 
					   int whichBlock);
OSLLIBAPI const double * OSLLINKAGE ekk_blockElement(EKKModel * model, 
					       int whichBlock);
OSLLIBAPI int OSLLINKAGE ekk_getInumblocks(EKKModel * model);
  /* these two are equivalent (no setInumels) */
OSLLIBAPI int OSLLINKAGE ekk_getInumels(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_sizeMatrix(EKKModel * model);
/* 1 if triplet, 2 if ordered */
OSLLIBAPI int OSLLINKAGE ekk_blockType(EKKModel * model, int whichBlock);

OSLLIBAPI int OSLLINKAGE  ekk_createRowCopy(EKKModel * model, EKKMatrixCopy *copy);
OSLLIBAPI int OSLLINKAGE  ekk_createColumnCopy(EKKModel * model, EKKMatrixCopy *copy);

OSLLIBAPI int OSLLINKAGE  ekk_deleteMatrixCopy(EKKMatrixCopy *copy);

  /*****
	The other flavor of matrix modification routines has been
	described before with functions such as ekk_addOneRow, but other
	useful ones are listed below. The most general is replaceElements
	which takes a list of matrix coefficients as triples and modifies, 
	inserts or deletes as appropriate; if an element value on list is zero
	then the corresponding element in matrix is deleted, if an element
	exists it will be modified and if it does not exist then it will be
	inserted.  The ekk_deleteXXXX functions take a list of rows or 
	columns to delete.  */
OSLLIBAPI int OSLLINKAGE ekk_replaceElements(EKKModel * model , 
			     int numberElements , const int * indexRow , 
			     const int * indexColumn , const double * element);
OSLLIBAPI int OSLLINKAGE ekk_addColumns(EKKModel * model , 
                     int numberToAdd , const double * objectiveCoefficient, 
                     const double * lowerBound , const double * upperBound ,
                     const int * startOfEachColumn , const int * indexRow , 
		     const double * element);
OSLLIBAPI int OSLLINKAGE ekk_deleteColumns(EKKModel * model, 
			       int numberToDelete , const int * which);
OSLLIBAPI int OSLLINKAGE ekk_replaceColumn(EKKModel * model , int iColumn ,
                   int numberElements,
                   const int * indexRow , const double * element);
OSLLIBAPI int OSLLINKAGE ekk_addRows(EKKModel * model , 
                   int numberToAdd , const double * lowerBound ,
                   const double * upperBound , const int * startOfEachRow ,
                   const int * indexColumn , const double * element);
OSLLIBAPI int OSLLINKAGE ekk_deleteRows(EKKModel * model, 
                    int numberToDelete , const int * which);
OSLLIBAPI int OSLLINKAGE ekk_replaceRow(EKKModel * model , int iRow ,
                   int numberElements,
                   const int * indexRow , const double * element);

  /*****
	More advanced model manipulation.  It is possible just to
	resize the model.  If rows or columns increases they get
	padded out with defaults, if a decrease occurs then arrays
	and matrix coefficients are truncated.  It is also possible
	and even useful to make a copy of all or part of a model.  A
	new model should be created and then ekk_copyModel or 
	ekk_copySubModel called. */
  /* if new is -1 then no change.  ifCompress switch to do things such as
       reallocate arrays on downsizing and changing maxrows */
OSLLIBAPI int OSLLINKAGE ekk_resizeModel(EKKModel * model , int newNumberRows ,
            int newNumberColumns ,  int ifCompress);
OSLLIBAPI int OSLLINKAGE ekk_copyModel(EKKModel * model ,
                          EKKModel * oldModel);
  /* ekk_copySubModel loses Quadratic and Presolve information */
  /* a row or column can appear twice in list and will be duplicated */
OSLLIBAPI int OSLLINKAGE ekk_copySubModel(EKKModel * model ,
              EKKModel * oldModel,int numberRows2,int numberColumns2,
              const int * selectedRows,const int * selectedColumns);

  /*****
	If you have multiple models then you may wish to do such things
	as changing a tolerance in all models.  using ekk_setXXXX on the
	base model defined in the OSL context variable will mean that
	will be the default for all subsequent new models.  Otherwise
	you can loop over all models setting values.  Also you may
	wish to attach user data to a particular model (or for callbacks)
	and this can be done using ekk_setUserData and passing a 
	void pointer */

OSLLIBAPI EKKModel * OSLLINKAGE ekk_baseModel(const EKKContext * context);
OSLLIBAPI int OSLLINKAGE ekk_numberOfModels(const EKKContext * context);
OSLLIBAPI EKKModel * OSLLINKAGE ekk_model(const EKKContext * context,
					    int whichModel);
OSLLIBAPI EKKContext * OSLLINKAGE ekk_context(const EKKModel * model);

OSLLIBAPI void * OSLLINKAGE ekk_userData(EKKModel * model); /* anything */
OSLLIBAPI int OSLLINKAGE ekk_setUserData(EKKModel * model,
                                    void * const userData); /* anything */

  /*****
	OSL has callbacks on particular messages and actions.  To register
	a callback at each iteration or factorization (see reference
	for details) register ItruCallBack while for messages use
	MsguCallBack.  There are others - see near end of header file.
	To activate a message call back for a particular message use
	ekk_messageUserExitOn. */

typedef  void (OSLLINKAGE_CB *ekkcbitru) (EKKModel * model,int reason, int * status);
OSLLIBAPI void OSLLINKAGE ekk_registerItruCallBack(EKKModel * model, 
						   ekkcbitru useritru);
  /* this sets frequency of callback for normal iterations */
OSLLIBAPI int OSLLINKAGE ekk_getIiterufreq(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setIiterufreq(EKKModel * model , int value);

typedef  void (OSLLINKAGE_CB *ekkcbmsgu) (EKKModel * model,int  msgno, int nreal,
                            double * rvec, int nint, int * ivec,
                            int nchar, char * cvec);
OSLLIBAPI void OSLLINKAGE ekk_registerMsguCallBack(EKKModel * model, 
						   ekkcbmsgu usermsgu);
OSLLIBAPI void OSLLINKAGE ekk_clearItruCallBack(EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_clearMsguCallBack(EKKModel * model);

OSLLIBAPI int OSLLINKAGE ekk_messageUserExitOn(EKKModel * model ,
					       int messgeNumber);
OSLLIBAPI int OSLLINKAGE ekk_messageUserExitOff(EKKModel * model ,
						int messageNumber);

  /*****
	More for Branch and Bound.  You can get more information on 
	integer variables and change information on sets - again
	be careful when using information returned by ekk_setInfo.
	For the array returned by ekk_listOfIntegers use ekk_free to
	delete.  You can delete all integer information as well. */
OSLLIBAPI int OSLLINKAGE ekk_deleteIntegerInformation(EKKModel * model);
OSLLIBAPI int * OSLLINKAGE ekk_listOfIntegers(EKKModel * model);
  /* or an array of column length 1=0/1 , 2=UI , 3=SOS2 , 0= not int */
OSLLIBAPI const char * OSLLINKAGE ekk_integerType(EKKModel * model);
  /* returns a structure which givesinfo on set and points
     to start of set (as array of EKKSetEntry structures) */
OSLLIBAPI EKKIntegerSet OSLLINKAGE ekk_setInfo(EKKModel * model, int iSet);
OSLLIBAPI EKKIntegerSet OSLLINKAGE ekk_getSOSInfo(EKKModel * model, int iSet);
  /* sets priority on set */
OSLLIBAPI int OSLLINKAGE ekk_setPriority(EKKModel * model, int iSet,
         int priority);
  /* no corresponding ekk_set for these two */
OSLLIBAPI int OSLLINKAGE ekk_getInumints(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getInumsets(EKKModel * model);

  /***** Here are some more useful tolerances etc.  If you want
	 more than a one-liner, please look at primer or reference */

  /* If a functions takes more than this print out CPU time */
OSLLIBAPI double OSLLINKAGE ekk_getRprintcpu(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setRprintcpu(EKKModel * model , double value);

  /* for simplex (and crossover after barrier) */
  /* maximum number of iterations (stop when ekk_getIiternum() >=) */
OSLLIBAPI int OSLLINKAGE ekk_getImaxiter(EKKModel * model);
  /* modify frequency and amount of print out */
OSLLIBAPI int OSLLINKAGE ekk_getIlogfreq(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIloglevel(EKKModel * model);
  /* change maximum number of iterations between factorizations */
OSLLIBAPI int OSLLINKAGE ekk_getImaxfactor(EKKModel * model);
  /* choose between devex and steepest edge pricing */
OSLLIBAPI int OSLLINKAGE ekk_getIdevexmode(EKKModel * model);
  /* get (only get) minor problem status */
OSLLIBAPI int OSLLINKAGE ekk_getIprobstat2(EKKModel * model);
  /* get primal and dual infeasibility tolerances */
OSLLIBAPI double OSLLINKAGE ekk_getRtolpinf(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRtoldinf(EKKModel * model);
  /* if perturbation occurs modify amount, if negative perturb at once */
OSLLIBAPI double OSLLINKAGE ekk_getRprimalPerturbation(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRdualPerturbation(EKKModel * model);

OSLLIBAPI int OSLLINKAGE ekk_setImaxiter(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIlogfreq(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIloglevel(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setImaxfactor(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIdevexmode(EKKModel * model , int value);
  OSLLIBAPI int OSLLINKAGE  ekk_setIprobstat2(EKKModel * model , int value);

OSLLIBAPI int OSLLINKAGE ekk_setRtolpinf(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRtoldinf(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRprimalPerturbation(EKKModel * model , 
						    double value);
OSLLIBAPI int OSLLINKAGE ekk_setRdualPerturbation(EKKModel * model , 
						  double value);

  /* for barrier */
  /* maximum number of barrier iterations */
OSLLIBAPI int OSLLINKAGE ekk_getImaxiterb(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setImaxiterb(EKKModel * model , int value);

  /* For branch and bound */
OSLLIBAPI int OSLLINKAGE ekk_getInumsols(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRbestsol(EKKModel * model);
  /* weight applied for each unsatisfied variable */
OSLLIBAPI double OSLLINKAGE ekk_getRiweight(EKKModel * model);
  /* amount by which a solution must improve (may be negative!) */
OSLLIBAPI double OSLLINKAGE ekk_getRimprove(EKKModel * model);
  /* set a target solution value */
OSLLIBAPI double OSLLINKAGE ekk_getRtarget(EKKModel * model);
  /* set a scale factor for all estimated degradations, if 0.0 computes
     to hit Rtarget. */
OSLLIBAPI double OSLLINKAGE ekk_getRdegscale(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setInumsols(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setRbestsol(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRiweight(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRimprove(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRtarget(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRdegscale(EKKModel * model , double value);
  /* Names of various MPS type artefacts - name is also printed
     out as model name */
OSLLIBAPI const char * OSLLINKAGE ekk_getCname(EKKModel * model);
OSLLIBAPI const char * OSLLINKAGE ekk_getCobjective(EKKModel * model);
OSLLIBAPI const char * OSLLINKAGE ekk_getCrhs(EKKModel * model);
OSLLIBAPI const char * OSLLINKAGE ekk_getCrange(EKKModel * model);
OSLLIBAPI const char * OSLLINKAGE ekk_getCbound(EKKModel * model);
OSLLIBAPI const char * OSLLINKAGE ekk_getCbasis(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setCname(EKKModel * model , const char * value);
OSLLIBAPI int OSLLINKAGE ekk_setCobjective(EKKModel * model , const char * value);
OSLLIBAPI int OSLLINKAGE ekk_setCrhs(EKKModel * model , const char * value);
OSLLIBAPI int OSLLINKAGE ekk_setCrange(EKKModel * model , const char * value);
OSLLIBAPI int OSLLINKAGE ekk_setCbound(EKKModel * model , const char * value);
OSLLIBAPI int OSLLINKAGE ekk_setCbasis(EKKModel * model , const char * value);
  /*****
	Miscellaneous functions.
  */
OSLLIBAPI int OSLLINKAGE ekk_allSlackBasis(EKKModel * model);
  /*****
	Functions which may be useful.  ekk_free is necessary to free
	any array returned by an ekk_xxxxx function */
  /* mallocs doubles */
OSLLIBAPI double * OSLLINKAGE ekk_double(int number_entries);
  /* mallocs ints */
OSLLIBAPI int * OSLLINKAGE ekk_int(int number_entries);
  /* frees an array */
OSLLIBAPI void OSLLINKAGE ekk_free(void * address);
  /* and some sorting */
OSLLIBAPI void OSLLINKAGE ekk_sortOnInt(int * key,
					 double * array,int number);
OSLLIBAPI void OSLLINKAGE ekk_sortOnDouble(double * key,
					 int * array,int number);
OSLLIBAPI void OSLLINKAGE ekk_sort(int * key,int number);
/****************************************************************/
/*     These are less used functions                            */
/*       see reference documentation for a description          */
/****************************************************************/

/* method fastInteger.  Sets extra bits to allow fast solves
   may be split up later for readability
   1 skip initial basis validity check
   2 skip non-basic value cleaning check
   4 skip back substitution error checking
   8 skip final invert if less than 20? iterations since factorization
   16&32 (0-3) number of factorizations to save */
OSLLIBAPI int OSLLINKAGE ekk_getIfastinteger(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setIfastinteger(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_getImaxfactorsize(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setImaxfactorsize(EKKModel * model , int value);
OSLLIBAPI double OSLLINKAGE ekk_getRobjectiveOffset(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setRobjectiveOffset(EKKModel * model , 
						 double value);
OSLLIBAPI int OSLLINKAGE ekk_isModelInteger(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_isModelQuadratic(EKKModel * model);
/* returns pivot row if basic, -1 if don't know row, -2 if not basic */
OSLLIBAPI int OSLLINKAGE ekk_isColumnBasic(EKKModel * model,int iColumn);
OSLLIBAPI int OSLLINKAGE ekk_columnStatus(EKKModel * model,int iColumn);
OSLLIBAPI int OSLLINKAGE ekk_markColumnsAsBasic(EKKModel * model,
                  int numberToMark , const int * which);
OSLLIBAPI int OSLLINKAGE ekk_setColumnStatus(EKKModel * model,
                  int numberToMark , const int * which, const int *usrstatus);
OSLLIBAPI int OSLLINKAGE ekk_setColumnsNonBasicAtLower(EKKModel * model,
                  int numberToMark , const int * which);
OSLLIBAPI int OSLLINKAGE ekk_setColumnsNonBasicAtUpper(EKKModel * model,
                  int numberToMark , const int * which);
OSLLIBAPI int OSLLINKAGE ekk_getNumberBasicColumns(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setRowStatus(EKKModel * model,
                  int numberToMark , const int * which, const int *usrstatus);
OSLLIBAPI int OSLLINKAGE ekk_markRowsAsBasic(EKKModel * model,
                  int numberToMark , const int * which);
OSLLIBAPI int OSLLINKAGE ekk_setRowsNonBasicAtLower(EKKModel * model,
                  int numberToMark , const int * which);
OSLLIBAPI int OSLLINKAGE ekk_setRowsNonBasicAtUpper(EKKModel * model,
                  int numberToMark , const int * which);
OSLLIBAPI int OSLLINKAGE ekk_getNumberBasicRows(EKKModel * model);
/* returns pivot row if basic, -1 if don't know row, -2 if not basic */
OSLLIBAPI int OSLLINKAGE ekk_isRowBasic(EKKModel * model,int iRow);
OSLLIBAPI int OSLLINKAGE ekk_rowStatus(EKKModel * model,int iRow);
  /* user can change bounds so int type changes - this checks */
OSLLIBAPI int OSLLINKAGE ekk_validateIntegers(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_mset(EKKModel * model ,int startNumber,
      int maxAllowed,int maxPrint,int userExit,int endNumber,
        int noNumber);
OSLLIBAPI int OSLLINKAGE ekk_addElementBlockWithOffsets(EKKModel * model , 
                  int numberElements ,
                  const int * indexRow , const int * indexColumn , 
		  const double * element,int rowOffset,
                  int columnOffset);
OSLLIBAPI int OSLLINKAGE ekk_addColumnElementBlockWithOffsets(
      EKKModel * model , int numberColumns,int baseColumn,int rowOffset,
                     const int * indexRow , const int * startOfEachColumn , 
                     const double * element);
OSLLIBAPI int OSLLINKAGE ekk_addSpaceForColumnElementBlock(EKKModel * model,
        int baseColumn, int numberColumns, int numberElements);
OSLLIBAPI int OSLLINKAGE ekk_repeatElementBlock(EKKModel * model , 
               int whichBlock, int rowOffset, int columnOffset, int ifCopy);
OSLLIBAPI int OSLLINKAGE ekk_dropLastElementBlock(EKKModel * model);
  /* this one allows fine tuning of spare space */
OSLLIBAPI int OSLLINKAGE ekk_resizeModelExact(EKKModel * model ,
            int newNumberRows , int newNumberColumns ,
            int maxRows , int maxCols ,  int ifCompress);
/*Function infeasibilites.
   when mask bit 8 is set:
     input:
       index has length 9.
       index[0] is lengh of table.
     output:
       if index[1]!=-1 then the index[1] to the index[2] entries of
         table contain the row numbers whose lower bound is a member of
         the IIS.
       if index[3]!=-1 then the index[3] to the index[4] entries of
         table contain the row numbers whose upper bound is a member of
         the IIS.
       if index[5]!=-1 then the index[5] to the index[6] entries of
         table contain the column numbers whose lower bound is a member of
         the IIS.
       if index[7]!=-1 then the index[7] to the index[8] entries of
         table contain the column numbers whose upper bound is a member of
         the IIS.    
*/
OSLLIBAPI int OSLLINKAGE ekk_infeasibilities(EKKModel * model ,
        int mask,int output,int * index,int * table);
/* resizes element block - for column ordered uses max columns */
/* will not resize below current numbers */
OSLLIBAPI int OSLLINKAGE ekk_resizeElementBlock(EKKModel * model , int iBlock ,
            int maximumNumberElements ,  int ifCompress);
/* checks all arrays not overwritten */
OSLLIBAPI int OSLLINKAGE ekk_validateModel(EKKModel * model); 
#ifndef EKK_THREADSAFE
OSLLIBAPI int OSLLINKAGE ekk_loadDspace(EKKModel * model, double * dspace ,
					int lengthOfDspace);
OSLLIBAPI int OSLLINKAGE ekk_unloadDspace(EKKModel * model , double * dspace);
#endif
#ifndef EKK_THREADSAFE
/*:method lpdc. */
OSLLIBAPI int OSLLINKAGE ekk_decomposeCrash(EKKModel * model,int i1,int i2,
					    int i3);
#endif
  /* mode =1 -> scale using existing scales, 2-> unscale */
  /* elements must be scaled before rim and unscaled after (if both) */
OSLLIBAPI int OSLLINKAGE ekk_scaleElements(EKKModel * model,int mode);
OSLLIBAPI int OSLLINKAGE ekk_scaleRim(EKKModel * model,int mode);
  /* returns 0 if no scaling, -1 if scaling factors, +1 if elements scaled
     and +2 if rim scaled as well */
OSLLIBAPI int OSLLINKAGE ekk_scaling(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_findSOS(EKKModel * model);
  /* The model is modified so if user wants to do fromBinary must copy model */
  /* type 1 = log, 2 = sos */
OSLLIBAPI int OSLLINKAGE ekk_makeBinary(EKKModel * model,int type);
OSLLIBAPI int OSLLINKAGE ekk_fromBinary(EKKModel * originalModel,
           EKKModel * binaryModel);
  /* y is modified and is of length numrows */
OSLLIBAPI int OSLLINKAGE ekk_matrixVector(EKKModel * model,double alpha ,
     const double * x, double beta , double * y);
  /* y is modified and is of length numcols */
OSLLIBAPI int OSLLINKAGE ekk_matrixTransposeVector(EKKModel * model,
						   double alpha ,
     const double * x, double beta , double * y);
  /* Note columns of basis taken as MATRIX order */
OSLLIBAPI int OSLLINKAGE ekk_formBInverseb(EKKModel * model, double  *x);
OSLLIBAPI int OSLLINKAGE ekk_formBInverseTransposeb(EKKModel * model, 
						    double  *x);
OSLLIBAPI int OSLLINKAGE ekk_createBInverse(EKKModel * model);

/* does factorization if needed */
OSLLIBAPI int OSLLINKAGE ekk_tableauArtificialColumn(EKKModel * model,
						     int iColumn, double zeroTolerance,
						     EKKVector *vec);

OSLLIBAPI int OSLLINKAGE ekk_tableauStructuralColumn(EKKModel * model,
						     int iColumn, double zeroTolerance,
						     EKKVector *vec);

/* puts result in user array (0-numberRows-1 rows, rest columns) */
OSLLIBAPI int OSLLINKAGE ekk_tableauRow(EKKModel * model, int iRow,
					double zeroTolerance,
					EKKVector *vec1,
					EKKVector *vec2);

OSLLIBAPI const double * OSLLINKAGE ekk_rowscales(EKKModel * model);
OSLLIBAPI const double * OSLLINKAGE ekk_colscales(EKKModel * model);

OSLLIBAPI EKKVector OSLLINKAGE ekk_getColumn(EKKModel * model , int iColumn);
OSLLIBAPI EKKVector OSLLINKAGE ekk_getRow(EKKModel * model , int iRow);

/* just calls ekk_free on the two component arrays */
OSLLIBAPI void OSLLINKAGE ekk_freeVector(EKKVector *vec);

OSLLIBAPI void OSLLINKAGE ekk_stderrNumber(EKKModel * model , 
					   int messageNumber);
OSLLIBAPI int OSLLINKAGE ekk_selectionList(EKKModel * model ,
 const char * selName, char wildcard, char arbchar,int oper, int rowcol);


OSLLIBAPI int OSLLINKAGE ekk_blockRowBase(EKKModel * model, int whichBlock);
OSLLIBAPI int OSLLINKAGE ekk_blockColumnBase(EKKModel * model, int whichBlock);
OSLLIBAPI int OSLLINKAGE ekk_blockNumberElements(EKKModel * model, int whichBlock);
OSLLIBAPI int OSLLINKAGE ekk_blockNumberColumns(EKKModel * model, 
				 int whichBlock);/* returns -1 if triplet */
OSLLIBAPI int OSLLINKAGE ekk_quadraticBlockType(EKKModel * model, 
                         int whichBlock);/* 1 if triplet, 2 if ordered */
OSLLIBAPI int OSLLINKAGE ekk_quadraticBlockRowBase(EKKModel * model, 
                               int whichBlock);
OSLLIBAPI int OSLLINKAGE ekk_quadraticBlockColumnBase(EKKModel * model, 
                             int whichBlock);
OSLLIBAPI int OSLLINKAGE ekk_quadraticBlockNumberElements(EKKModel * model, 
                               int whichBlock);
OSLLIBAPI int OSLLINKAGE ekk_quadraticBlockNumberColumns(EKKModel * model, 
                        int whichBlock);/* returns -1 if triplet */
OSLLIBAPI const int * OSLLINKAGE ekk_quadraticBlockRow(EKKModel * model, 
                          int whichBlock);
OSLLIBAPI const int * OSLLINKAGE ekk_quadraticBlockColumn(EKKModel * model, 
                           int whichBlock);
OSLLIBAPI const double * OSLLINKAGE ekk_quadraticBlockElement(EKKModel * model, 
                           int whichBlock);
OSLLIBAPI int OSLLINKAGE ekk_senseBound(EKKModel * model, 
                           int type, int respect);
OSLLIBAPI int OSLLINKAGE ekk_senseObjective(EKKModel * model); 
/* returns pivot row (only useful if dual infeasible) */
OSLLIBAPI int OSLLINKAGE ekk_pivotRow(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_sequenceOut(EKKModel * model);
/* activates or deactivates cutoff etc */
OSLLIBAPI int OSLLINKAGE ekk_activateCutoff(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_deactivateCutoff(EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_printBase(EKKModel * model, int base);
OSLLIBAPI int OSLLINKAGE ekk_getInumunsat(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setImaxrows(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setImaxcols(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIoneobj(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIstopmask(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIpossbasis(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setImsgpos(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIpagelines(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIprtmtrxmask(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIqparmaxiter(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIintmask(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIfastits(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setInumunsat(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setImajorits(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIprintsens(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIstrategy(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIthreshold(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIheurpass(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIsupertol(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIroword(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIordunit(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setImiplength(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIiterbnum(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIpricetype(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIlpdcflag(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setRpweight(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRchangeweight(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRdweight(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRtolmps(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRtolint(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRbestposs(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRbestest(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRdensethr(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRpertdiag(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRnetsamp(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRmulinfac(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRthreshold(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRintegerGap(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_getIoneobj(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIqparnumiter(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getImaxrows(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getImaxcols(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIstopmask(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIpossbasis(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getInumquadraticblocks(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getImsgpos(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIpagelines(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIprtmtrxmask(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIqparmaxiter(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getInumnodes(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIintmask(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIfastits(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getImajorits(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIprintsens(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIstrategy(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIthreshold(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIheurpass(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIsupertol(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIroword(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIordunit(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getImiplength(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIiterbnum(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIpricetype(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getInumelq(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIlpdcflag(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRmulinfac(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRpweight(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRchangeweight(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRdweight(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRtolmps(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRtolint(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRbestposs(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRbestest(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRdensethr(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRthreshold(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRintegerGap(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRpertdiag(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRnetsamp(EKKModel * model);
  /* at present only meaningful after call to ekk_xxxxxSimplex 
     1 = primal, 2 = dual */
OSLLIBAPI int OSLLINKAGE ekk_lastAlgorithm(EKKModel * model);

  /* return user_returncode as for ekkbrnu */

typedef  int (OSLLINKAGE_CB *ekkcbbrnu) (EKKModel * model,int reason,
                            int * iarray, double * rarray);
OSLLIBAPI void OSLLINKAGE ekk_registerBrnuCallBack(EKKModel * model, ekkcbbrnu userbrnu);

typedef  int (OSLLINKAGE_CB *ekkcbchnu) (EKKModel * model,int reason,
                            int * iarray, double * rarray);
OSLLIBAPI void OSLLINKAGE ekk_registerChnuCallBack(EKKModel * model, ekkcbchnu userchnu);
  /* return user_returncode as for ekkevnu */

typedef  int (OSLLINKAGE_CB *ekkcbevnu) (EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_registerEvnuCallBack(EKKModel * model, ekkcbevnu userevnu);

typedef  void (OSLLINKAGE_CB *ekkcbnodu) (EKKModel * model, int reason, int * iarray,
                            double * rarray, int * muser , int nuser);
OSLLIBAPI void OSLLINKAGE ekk_registerNoduCallBack(EKKModel * model, ekkcbnodu usernodu);

typedef  void (OSLLINKAGE_CB *ekkcbslvu) (EKKModel * model, int reason,
                            double * objective_value ,  int * status);
OSLLIBAPI void OSLLINKAGE ekk_registerSlvuCallBack(EKKModel * model, ekkcbslvu userslvu);

typedef  void (OSLLINKAGE_CB *ekkcbordu) (EKKModel * model, int * xadj,
                            int * adjncy , int * perm , int * invp,
                            int * mwork, int nwords, int nrow);
OSLLIBAPI void OSLLINKAGE ekk_registerOrduCallBack(EKKModel * model, ekkcbordu userordu);
OSLLIBAPI void OSLLINKAGE ekk_clearBrnuCallBack(EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_clearChnuCallBack(EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_clearEvnuCallBack(EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_clearNoduCallBack(EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_clearSlvuCallBack(EKKModel * model);
OSLLIBAPI void OSLLINKAGE ekk_clearOrduCallBack(EKKModel * model);
  /* below here may not be documented at all */

  /* if users wish some of these can be mapped to OSL parameters */
typedef struct{
  /* terminating values */
  double alpha;
  double lambda;
  double lcost;
  /* parameters */
  double lambdaInit;/* default 0.01 */
  double alphaInit;/* default 0.01 */
  double alphaFactor;/* default 0.5 */
  double alphaMin;/* default 0.001 */
  double primalAbsolutePrecision;
  double gapAbsolutePrecision;
  double gapRelativePrecision;
  double granularity;
  double minimumRelativeAscent;
  int ascentFirstCheck;
  int ascentCheckInterval;
  int printFlag; /* default 1 */
  int printInterval; /* default 100 */
  int greenTestInterval; /* default 2 */
  int yellowTestInterval;/* default 2 */
  int redTestInterval;/* default 10 */
  int alphaInt; /* default 50 */
  int maxSubGradientIterations; /* default 2000 */
  /* nonexistent in VOL_parm */
  int method; /* set by ekk_checkValidVolume */
  int useDualData; /* set by ekk_checkValidVolume */
} EKKVolumeInfo;

typedef struct{
  double djTolerance;
  double mu;  /* starting mu */
  double drop; /* exit if drop over 5 checks less than this */
  double muFactor; /* reduce mu by this */
  double stopMu; /* exit if mu gets smaller than this */
  double smallInfeas; /* feasibility tolerance */
  double reasonableInfeas; /* use lambdas if feasibility less than this */
  double exitDrop; /* candidate for stopping after a major iteration */
  double muAtExit; /* mu on exit */
  double exitFeasibility; /* exit if infeasibility less than this */
  double dropEnoughFeasibility; /* okay if feasibility drop this factor */
  double dropEnoughWeighted; /* okay if weighted obj drop this factor */
  int maxBigIts; /* always reduce mu after this */
  int maxIts; /* do this many iterations on first go */
  int logLevel;
  int logFreq;
  int checkFrequency; /* can exit after 5 * this iterations (on drop) */
  int lambdaIterations; /* do at least this many lambda iterations */ 
  int maxIts2; /* do this many iterations on subsequent goes */
  int addAll;    /* How many steps the simplex crossover should take to
		     arrive to an extreme point:
		     0 - chosen,all used, dj tolerance ,all 
		     1 - chosen,all used, all
		     2 - chosen, all
		     3 - all
		     4 - do not do anything (no simplex)
                     8 (added) do crash to get initial basis
		  */
  int strategy;   /* 0 - default strategy
		     1 - do accelerator step but be cautious
		     2 - do not do accelerator step 
		     4 - drop, exitDrop and djTolerance all relative
		     8 - keep accelerator step to theta=10.0
		    16 - scale infeasibilities using rhs 
                    32 - "intelligent?" reduction of mu and reasonableInfeas
                  2048 - keep lambda across mu change
		  4096 - return best solution (not last found) */
} EKKIdiotInfo;

   /* Initialize default values for the idiot code. This could be done
      in ekk_initialize() */
  /* this is now done by ekk_idiotInfo if necessary */
   OSLLIBAPI EKKIdiotInfo * OSLLINKAGE ekk_initializeIdiot(EKKModel * model);
  /* returns idiot code structure */
   OSLLIBAPI EKKIdiotInfo * OSLLINKAGE ekk_idiotInfo(EKKModel * model);
  /* Solve the LP with the idiot code */
   OSLLIBAPI int OSLLINKAGE ekk_idiot(EKKModel * model);
   OSLLIBAPI int OSLLINKAGE ekk_idiotCrossOver(EKKModel * model);

/****************************************************************/
/*     These are obsolete or don't work                         */
/*     or are  experimental to some degree                      */
/*     or are for developers who can contact IBM                */
/****************************************************************/
OSLLIBAPI int OSLLINKAGE ekk_initialize();
OSLLIBAPI int OSLLINKAGE ekk_end(); /* document ? */
OSLLIBAPI int OSLLINKAGE ekk_deleteModel(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_convertMPS(EKKModel * model , const char * fileName);
OSLLIBAPI int OSLLINKAGE ekk_convertMPSFree(EKKModel * model , 
					    const char * fileName);
OSLLIBAPI EKKModel *  OSLLINKAGE ekk_createModel(const char * name);
OSLLIBAPI int OSLLINKAGE ekk_convertQuadratic(EKKModel * model , const char * fileName);
OSLLIBAPI int OSLLINKAGE ekk_loadQuadraticMatrix(EKKModel * model ,
            int type ,
           int  numberElements, const int * row, const int * column,
           const double * element);

typedef struct {
                 EKKModel ** subModel; /* includes master */
                 int ** masterRows;
                 int ** masterColumnStarts;
                 double ** masterElements;
                 int * rowBase;
                 int * columnBase;
                 int * toOriginalRows;
                 int * toOriginalColumns;
                 int numberBlocks;
                 int numberMasterRows;
                 int numberMasterColumns;
         } EKKDecomposed;
/*:method ekk_subModels */
OSLLIBAPI EKKDecomposed OSLLINKAGE ekk_subModels(EKKModel * model,
        int maxBlock);
OSLLIBAPI void OSLLINKAGE ekk_deleteSubModels(EKKModel * model,
                                              EKKDecomposed decomposition,
                                              int ifCopy);


OSLLIBAPI int OSLLINKAGE ekk_startFastDualSimplex(EKKModel * model, 
						  int maxIterations);
  /* up to user to restore basis */
/* mode bit 1 modify costs, bit 2 nonzero for dual, 
   bit 4 use sprint on clean up*/
OSLLIBAPI int OSLLINKAGE ekk_sprintCrossover(EKKModel * model,int mode,
					     int keep, double fixPrimal,
					     double fixDual);
OSLLIBAPI int OSLLINKAGE ekk_fastDualSimplex(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_stopFastDualSimplex(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setI(EKKModel * model, int value,
      int controlVariableNumber);
OSLLIBAPI int OSLLINKAGE ekk_setR(EKKModel * model, double value,
      int controlVariableNumber);
OSLLIBAPI int OSLLINKAGE ekk_setC(EKKModel * model, const char * value,
      int controlVariableNumber);
OSLLIBAPI int OSLLINKAGE ekk_getI(EKKModel * model,
              int controlVariableNumber);
OSLLIBAPI double OSLLINKAGE ekk_getR(EKKModel * model,
              int controlVariableNumber);
OSLLIBAPI const char * OSLLINKAGE ekk_getC(EKKModel * model,
              int controlVariableNumber);
  /* printing preference (first parameter also alters print for message 317)*/
  /* first parameter
                     0 - only print non get calls
                     1 - print all calls
     second parameter
                     0 - replace EKK by ekk_ (50 off)
                     1 - append EKK name to ekk_ name (50 on)
     third parameter
                     -1  - do not print entering parameters
                     >=0 - print entering parameters and arrays of length<parameter*/
OSLLIBAPI void OSLLINKAGE ekk_setPrinting(EKKModel * model, int preference,
                         int ifAppend, int ifPrintParameters);
OSLLIBAPI int OSLLINKAGE ekk_loadParametricData(EKKModel * model ,
    const double * objectiveDelta,
    const double * lowerRowDelta , const double * upperRowDelta ,
    const double * lowerColumnDelta ,  const double * upperColumnDelta);
OSLLIBAPI double * OSLLINKAGE ekk_objectiveDelta(EKKModel * model);
OSLLIBAPI double * OSLLINKAGE ekk_lowerRowDelta(EKKModel * model);
OSLLIBAPI double * OSLLINKAGE ekk_upperRowDelta(EKKModel * model);
OSLLIBAPI double * OSLLINKAGE ekk_lowerColumnDelta(EKKModel * model);
OSLLIBAPI double * OSLLINKAGE ekk_upperColumnDelta(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_doParametrics(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_primalQuadraticSimplex(EKKModel * model, int mode);
OSLLIBAPI int OSLLINKAGE ekk_dualQuadraticSimplex(EKKModel * model, int mode);
OSLLIBAPI int OSLLINKAGE ekk_quadraticCrossover(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_saveToFile(EKKModel * model ,  const char * fileName);
OSLLIBAPI int OSLLINKAGE ekk_restoreFromFile(EKKModel * model ,  const char * fileName);
OSLLIBAPI int OSLLINKAGE ekk_outMatrix(EKKModel * model, const char * fileName ,
              int ifCharacter ,  int numberFields);
OSLLIBAPI int OSLLINKAGE ekk_outBasis(EKKModel * model, const char * fileName ,
                   int ifCharacter);
OSLLIBAPI int OSLLINKAGE ekk_inBasis(EKKModel * model, const char * fileName);
OSLLIBAPI int OSLLINKAGE ekk_addNonLinear(EKKModel * model , int sequence ,
       int numberSegments ,
       const double * lower , const double * upper,
       const double * lowerCost , const double * upperCost);
OSLLIBAPI int OSLLINKAGE ekk_activateNonLinear(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_deactivateNonLinear(EKKModel * model);
OSLLIBAPI double * OSLLINKAGE ekk_rweight(EKKModel * model); /* devex weight row information*/
OSLLIBAPI double * OSLLINKAGE ekk_cweight(EKKModel * model); /* devex weight column information*/
OSLLIBAPI int OSLLINKAGE ekk_copyRowWeights(EKKModel * model,
                const double * fromArray, int toStart, int toEnd);
OSLLIBAPI int OSLLINKAGE ekk_addColumnElements(EKKModel * model , 
                   int numberElements,
                   const int * indexRow , const double * element);
OSLLIBAPI int OSLLINKAGE ekk_addRowElements(EKKModel * model , int numberElements,
                   const int * indexColumn , const double * element);
  /* old way of getting names */
OSLLIBAPI char   * OSLLINKAGE ekk_rownames(EKKModel * model);
OSLLIBAPI char   * OSLLINKAGE ekk_colnames(EKKModel * model);
OSLLIBAPI int * OSLLINKAGE ekk_sellistcol(EKKModel * model);
OSLLIBAPI int * OSLLINKAGE ekk_sellistrow(EKKModel * model);
OSLLIBAPI int * OSLLINKAGE ekk_arcid(EKKModel * model);
OSLLIBAPI int * OSLLINKAGE ekk_level(EKKModel * model);
OSLLIBAPI int * OSLLINKAGE ekk_parent(EKKModel * model);
OSLLIBAPI int * OSLLINKAGE ekk_preorder(EKKModel * model);
OSLLIBAPI int * OSLLINKAGE ekk_revpreorder(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_setInullcheck(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIextrablk(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIdroprowct(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setItotalcpu(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIwhichcpu(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIstripes(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setInumcpu(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setImaxprojns(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIsmdltypemask(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setIordthrsh(EKKModel * model , int value);
OSLLIBAPI int OSLLINKAGE ekk_setRfixvar1(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRfixvar2(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRcholabstol(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRcholtinytol(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRcholreltol(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRprojtol(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRstepmult(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRlambdaval(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRdccutoff(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRdobjval(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRslambda(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRslambdalim(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRslambdadelta(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_setRpdstepmult(EKKModel * model , double value);
OSLLIBAPI int OSLLINKAGE ekk_getInullcheck(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIdroprowct(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIextrablk(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getInumcpu(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getItotalcpu(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIwhichcpu(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIstripes(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getImaxprojns(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIsmdltypemask(EKKModel * model);
OSLLIBAPI int OSLLINKAGE ekk_getIordthrsh(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRfixvar1(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRfixvar2(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRcholabstol(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRcholtinytol(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRcholreltol(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRprojtol(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRstepmult(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRlambdaval(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRdccutoff(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRdobjval(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRslambda(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRslambdalim(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRslambdadelta(EKKModel * model);
OSLLIBAPI double OSLLINKAGE ekk_getRpdstepmult(EKKModel * model);
OSLLIBAPI int * OSLLINKAGE ekk_long(int number_entries);
/* for sensitivity information */
typedef struct {
  /* for objective */
  const double * sobjupc; /* upward costs */
  const double * sobjdnc; /* downward costs */
  const double * sobjupv; /* upward change in objective value */
  const double * sobjdnv; /* downward change in objective value */
  const int * sobjupl; /* sequence of leaving variable (upward) */
  const int * sobjdnl; /* sequence of leaving variable (downward) */
  const int * sobjupe; /* */
  const int * sobjdne; /* */
  
  /* for bounds */
  const double * sbndrupb;
  const double * sbndrdnb;
  const double * sbndrupv;
  const double * sbndrdnv;
  const int * sbndrupe;
  const int * sbndrdne;
  const int * sbndrupl;
  const int * sbndrdnl;
  const double * sbndcupb;
  const double * sbndcdnb;
  const double * sbndcupv;
  const double * sbndcdnv;
  const int * sbndcupe;
  const int * sbndcdne;
  const int * sbndcupl;
  const int * sbndcdnl;
} EKKSensitivityInfo;
OSLLIBAPI EKKSensitivityInfo OSLLINKAGE ekk_sensitivityInfo(EKKModel * model);
  /************ start of integer presolve stuff */
  typedef struct {
    unsigned int binary_at_0_or_1 : 1; 
    unsigned int affects_which_bound : 1; /* i.e. fixes to */
    unsigned int variable : 30; /* variable affected */ 
  } EKKIntegerInformation;
  
  typedef struct {
    /* Number of 0-1 variables */
    int number01;
    /* Real sequence numbers of 0-1 variables */
    int * sequence;
    /* Number of cliques */
    int numberClique;
    /* a clique is a set of variables such that setting any single member 
       one way fixes all the other members.  No information is recorded on
       the effect of fixing the member the other way.  The simplest clique
       is an SOS e.g. x + y + z <= 1 where fixing any one to 1 fixes others
       to 0.  However x - y + z <=0 is also a clique as x => 1 implies
       y => 1 and z => 0 while y => 0 implies x => 0, z => 0 and z => 1 is
       similar to x => 1.  The implied RHS is 1 - the number of -1s */
    /* clique types -   1 if equality, 2 if slack */
    int * cliqueType;
    /* In this structure the binary_at_0_or_1 field of structure signals
       whether setting a variable to 0 or 1 fixes all.  
       The affects_which_bound field is not used */
    EKKIntegerInformation * cliqueMember;
    /* This array gives the start of each clique */
    int * cliqueStart;
    /* Implication starts as number01 starts+1 - gives implications
       for each 0-1 variable */
    int * implicationStart;
    /* A 0-1 to 0-1 implication is a mini-clique so on normal exit all
       such will have been gathered into cliques or used to strengthen cliques,
       so this may be empty.  If there are continuous variables implications
       may still exist.  An implication is 0-1 variable at 0/1 forces
       another variable to be at lower/upper bound.  This will also
       include disaggregation information (which is normally phrased the
       other way round). */
    EKKIntegerInformation * implication;

    /* Chaining information for each 0-1 variable. This is like a more
       powerful form of clique in that setting any member fixes all others.
       This means that the set of variables can be treated as a single
       variable which leads to better pre-processing.  It is fairly
       obvious that no variable can belong to more than one "chain".
       Each 0-1 variable is marked with 0 (if not chained) or a chain
       number.  The sign used says which way a variable will go.  If x and
       y both have -2 then they are in the same chain and go to 0 or 1 
       together (i.e. x=y).  The same would be true if they both had +2 but
       if one had -2 and the other +2 then they go to different bounds i.e.
       x+y=1 */
    int numberChains; 
    int * chainInformation;
  }  EKKIntegerPresolve;
  typedef struct {
    double lowerBound;
    double upperBound;
    int numberElements;
    unsigned int cutsOffSolution:1; /* e.g. benders cut */
    unsigned int localCut:1;
    unsigned int doNotDelete:1; /* never delete cut from pool */
    unsigned int dummy:1;
    unsigned int extraInfo:28;
    int * variable;
    double * coefficient;
  } EKKCut;

  typedef struct {
    int numberCuts;
    int maxCuts;
    EKKCut * cut;
  } EKKCuts;
  
  /* Integer presolve:
     If the EKKIntegerPresolve parameter is null then no information
     is returned.  
     If the EKKCuts parameter is null then the model is updated
     by all cuts and coefficient strengthening.  If not null then 
     if cutMode is: 
       0 then the model is not updated and the cut information returned.
       1 then the cuts are used as input and the model is updated.
       2 then the cuts are used as input, updated and the model is not. 
       3 then EKKIntegerPresolve just sets up structures 
     returns 2 if optimal solution found, 
             1 if no possible solution,
             3 if there were no 0-1 integers, 
             4 if best posible solution worse than cutoff
             0 otherwise */
  OSLLIBAPI int OSLLINKAGE ekk_integerPresolve(EKKModel * model,
					       EKKIntegerPresolve * info,
					       EKKCuts * cuts,
					       int cutMode);
  /* gives local information inside branch and cut */
  OSLLIBAPI EKKIntegerPresolve * OSLLINKAGE 
                 ekk_integerPresolveInformation(EKKModel * model);
  OSLLIBAPI void OSLLINKAGE ekk_deleteIntegerPresolve(
					       EKKIntegerPresolve * info);
  OSLLIBAPI void OSLLINKAGE ekk_deleteCuts(EKKCuts * cuts);
  /* below is for use inside user cut generator */
  OSLLIBAPI EKKCut * OSLLINKAGE ekk_createOneCut(EKKCuts * cuts,
					     int number,
					     double lower, double upper,
					     const int * indices,
					     const double * coefficients);
  /* this is a short term solution until better cut pools etc */
  /* maxCuts is number at any one time */
  OSLLIBAPI int OSLLINKAGE ekk_branchAndCut(EKKModel * model,
					    const char * matrixFile ,
					    const char * basisFile,
					    EKKIntegerPresolve * info,
					    EKKCuts * cuts, int maxCuts,
					    int checkAtDepth);
  /* reason 0 - branchAndCut, 1 - integerPresolve, 2 - both */
  typedef  int (OSLLINKAGE_CB *ekkcbcutu) (EKKModel * model,int reason,
			   EKKIntegerPresolve * info,
			   EKKCuts * cuts, int depth);
			   
OSLLIBAPI void OSLLINKAGE ekk_registerCutuCallBack(EKKModel * model, ekkcbcutu usercutu);
OSLLIBAPI void OSLLINKAGE ekk_clearCutuCallBack(EKKModel * model);
  /*******  end integer presolve stuff */
/*:method set exotic parameters. */
OSLLIBAPI int OSLLINKAGE ekk_exoticParameters(EKKModel * model ,
       int numberLong, const int * iparam , int numberDouble, const double * dparam,
       int numberTolerances, const double * tolerances);
  /* number between 1 and 10 */
OSLLIBAPI void OSLLINKAGE ekk_userMessage(EKKModel * model,
					  int number,const char * string);

#ifdef __cplusplus
          }
#endif
#endif
