/*___________________________________________________________________________*/
/* IBM Optimization Solutions -- Stochastic Solutions/Extensions             */
/* scn_c_api.h -        C-API header file                                    */
/* (c) IBM Corp, 1999, 2000 All rights reserved.                             */
/*                                                                           */
/* US Government Users Restricted Rights - Use, duplication, or disclosure   */
/* restricted by GAS ADP Schedule Contract with IBM Corp.                    */
/*___________________________________________________________________________*/



/* Linker Decorations */
#ifdef __cplusplus
extern "C"{
#endif

#ifndef SCN_C_API
#define SCN_C_API

#ifndef OSLLIBAPI
#if defined (OSLMSDLL)
 #define OSLLIBAPI __declspec(dllexport)
 #define OSLLINKAGE  __stdcall
 #define OSLLINKAGE_CB __cdecl
#elif defined (OSLMSSTAT)
 #define OSLLINKAGE  __stdcall
 #define OSLLIBAPI
 #define OSLLINKAGE_CB  __cdecl
#else
 #define OSLLIBAPI
 #define OSLLINKAGE
 #define OSLLINKAGE_CB
#endif /*OSLMSDLL*/
#endif /*OSLLIBAPI*/

#include "ekk_c_api.h"
#if !defined(EKKSTOCH)
#define EKKSTOCH
typedef struct EKKStoch EKKStoch;

#endif


/* OSL Stochastic Solutions C-API functions */
/****************************************************************/
/*     These are the most important functions                   */
/*     note that non-zero return codes indicate some error      */
/*     The notes here are an abbreviated version of those in    */
/*     the (mythical) OSLSE primer                              */ 
/*     (this file is dedicated to David L. Jensen so that he    */
/*     he does not have to read any real documentation)         */
/*     ** If you think something needs moving/clarifying then   */
/*     inform Alan King                                         */
/****************************************************************/

  /*****
  0) this gives access to all OSL library functions */

  /*****
  1) a context for OSL has to be set up.  This is done just once in a
     program.  The context pointer is passed whenever a new model is
     created.  It is also passed to close down the context */
	 
  /* sets up entire OSL context and returns the context pointer */
OSLLIBAPI EKKContext *  OSLLINKAGE ekks_initializeContext();

  /* deletes entire OSL context.  All existing models will be deleted
     and any other areas OSL has created - including the context itself
     so there is no need to free "context" */
OSLLIBAPI void OSLLINKAGE ekks_endContext(EKKContext * context);

  /*****
  2) within the OSL context there can be as many stochastic models 
     as the user needs.  For each new stochastic model the OSL context 
	 is passed and an optional name, together with a parameter 
	 indicating the largest number of scenarios that the
     stochastic model can hold. */

  /* create a stochastic model with maximum "maxscn" scenarios */
OSLLIBAPI EKKStoch * OSLLINKAGE  ekks_newStoch(EKKContext *env,
				const char *modelname, int maxscn);

  /* delete a stochastic model - all memory used is freed */
OSLLIBAPI int OSLLINKAGE  ekks_deleteStoch(EKKStoch *stoch);

  /*****
  3) the easiest way to populate a stochastic model with data is 
     to use the Stochastic MPS (SMPS) format.
	 It consists of 
	  a) the "Core file" --- an MPS file that sets the "template". 
	     ...think of the core file as being the LP with all stochastic 
		 data set to their expected values.
	  b) the "Time file" --- which points to the beginning row and 
	     column for each time-stage. 
	  c) the "Stoch file" --- which contains a description of the 
	     stochastic data that replaces or adds to values in the Core. 

     For a description of the SMPS format implemented in OSLSE, see
     http://www-3.ibm.com/software/data/bi/osl/pubs/Stochsol/node2.htm#SECTION00220
	*/


  /* read SMPS data and return type of distribution in stoch file */
OSLLIBAPI int OSLLINKAGE  ekks_readSMPSData(EKKStoch *stoch, 
			const char *corefile, 
			const char *timefile, 
			const char *stochfile);

  /* these are the distribution types in case you're interested */
#define DISTN_NOT_IMPLEMENTED   0
#define SCENARIOS               1
#define INDEP_DISCRETE          2
#define BLOCK_DISCRETE          3

  /*****
  4) There are several algorithms available to solve the model.
     Stochastic programs have a nice structure, so we like to
     use decomposition solvers. */

  /* Solve the stochastic program using two-level Benders decomposition. */
OSLLIBAPI int OSLLINKAGE  ekks_bendersLSolve(EKKStoch *stoch, int startmode);

  /* here are the return codes */

#define INCOMPLETE 0200
#define STALLED    0100
#define STOPPED    040
#define FINISHED   020
#define IMPROPER   010
#define UNBOUNDED  004
#define WORKING    002
#define INFEASIBLE 001
#define OPTIMAL    000

  /*****
  5) After solving the model we need to get the results.  The best way to 
     do this is to use OSL's functions.  First we get the OSL model
	 pointer.*/

  /* get OSL model pointer */
OSLLIBAPI EKKModel * OSLLINKAGE  ekkse_getCurrentModel(EKKStoch *stoch);

  /************************************************************************/
  /************************************************************************

	That is enough to get useful work done.  Here is a valid OSLSE fragment 
    {
	  EKKContext * context = ekks_initializeContext();
	  EKKStoch * stoch = ekks_newStoch(context," ");
	  ekks_readSMPSData(stoch,"app0110.core","app0110.time","app0110.stoch");
	  ekks_bendersLSolve(stoch,0);
	  EKKModel * model = ekkse_getCurrentModel(stoch); 
	  ekk_printSolution(model);
	  ekk_endContext(context);
    }
  **************************************************************************
	
    The contents of the next sections are:

    6) Other ways to build a stochastic model.  
    7) Some other decomposition methods and ways to tune for them.
    8) Message handling, printing and accessing data
    9) Stochastic Mixed Integer Problems

       That may be enough for your first applications.  When you feel
       confident then look later sections (and in the documentation) 
       for more advanced features.  At the end there are prototypes for
       features which are only in the reference guide.    */

  /*****
  6) Other ways to build a model.
     a) OSLSE Native Data files
     b) Pass data using in-core-memory methods: 
			ekks_createCore, ekks_addScenario, ekks_changeProbability
		(see example exsgbd.c)
	 c) EKKStoch methods to build scenario trees
	    (see example exsgbdt.c)										*/

  /*** a) Read or write using OSLSE Native Data model (very fast) */
OSLLIBAPI int OSLLINKAGE  ekks_readNativeData(EKKStoch *stoch, 
		char *splfilename);
OSLLIBAPI int OSLLINKAGE  ekks_writeNativeData(EKKStoch *stoch,
		char *splfilename);

  /*** b) Pass data using in-core-memory methods:					*/

  /* Pass CORE model using in-core-memory data */
OSLLIBAPI int OSLLINKAGE  ekks_createCore(EKKStoch *stoch,
		int nstg, int *rstag, int *cstag, /* stage info				*/
		int irow, int icol, int iels,     /* # rows, cols, elements */
		double *dobj,                     /* objective				*/
		double *drlo, double *drup,       /* row bounds				*/
		double *dclo, double *dcup,       /* col bounds				*/
		int *mr, int *mc, double *dels,   /* matrix triples			*/
		double *dqdg);					  /* diag quadratic terms	*/
  

#ifndef CORETYPE_H
#define CORETYPE_H
        typedef enum {linear=1, quadratic=2} CoreType;
#endif

 /* Set CORE problem  type to LP or QP */
OSLLIBAPI int OSLLINKAGE  ekks_setCoreProblemType(EKKStoch *stoch,
		CoreType type);


#define REPLACE_CORE_VALUES     1  /* scenario values replace core */
#define ADD_TO_CORE_VALUES      0  /* scenario values add to core */

  /* Define Scenario using in-core-memory data */
OSLLIBAPI int OSLLINKAGE ekks_addScenario(EKKStoch *stoch,
		int nscn,						  /* scenario number */
		int ianc,						  /* # of ancestor scenario */
		int istg,						  /* branching stage */
		double dprob,					  /* probability */
		int itype,						  /* matrix type (not used) */
		int nrow, int ncol, int nels,
		double *dobj, 
		double *drlo, double *drup, 
		double *dclo, double *dcup,
		int *mrow, int	*mcol, double *dels,
		int replace 					  /* replace = ADD_TO_CORE_VALUES
										     or        REPLACE_CORE_VALUES */
		);
  /* Define Scenario as Block  */
OSLLIBAPI int OSLLINKAGE ekks_addScenarioBlock(EKKStoch *stoch,
		int nscn,						  /* scenario number */
		int ianc,						  /* # of ancestor scenario */
		int istg,						  /* branching stage */
		double dprob,					  /* probability */
		int itype,						  /* matrix type (not used) */
		int nrow, int ncol, int nels,		
		int *mrow,						/* row indices 
										   row lower bound ~ ncol+1
										   row upper bound ~ ncol+2 */
		int *mcol,						/* col indices 
										   col lower bound ~ nrow+1
										   col upper bound ~ nrow+2
										   objective	   ~ nrow+3 */
		double *dels,
		int replace 					  /* replace = ADD_TO_CORE_VALUES
										     or        REPLACE_CORE_VALUES */
		);


   /* Define BLOCK DISCRETE random variable using in-core-memory data.*/
OSLLIBAPI int OSLLINKAGE ekks_addIndependentBlock(EKKStoch *stoch, 
		int istg,						/* stage */
		int nevt,						/* number of discrete events */
		double *dprob,					/* array of event probabilities */
		int *evst,						/* array of event starts in matrix
										   evst[n] points to the matrix element
										   that starts the n-th event.
										   evst[nevt] points to the last entry.
										   */
		int *mrow,						/* row indices 
										   row lower bound ~ ncol+1
										   row upper bound ~ ncol+2 */
		int *mcol,						/* col indices 
										   col lower bound ~ nrow+1
										   col upper bound ~ nrow+2
										   objective	   ~ nrow+3 */
		double *dels);					/* matrix elements */

   /* After adding all BLOCK DISCRETE random variables, generate scenario tree */
OSLLIBAPI int OSLLINKAGE ekks_generateScenariosFromIndependentBlocks(EKKStoch *stoch, int replace);

  /* Reset probability weights */
OSLLIBAPI int OSLLINKAGE ekks_changeProbability(EKKStoch *stoch, double *dp);

  /* Replace tree with samples from original scenario tree
     samplemode =0 - uniform random sampling (the only supported mode) */
OSLLIBAPI int OSLLINKAGE  ekks_generateSamples(EKKStoch *stoch,
		int numsamples, int samplemode);

  /* Describe full or partial model to OSLSE */
OSLLIBAPI int OSLLINKAGE  ekks_describeFullModel(EKKStoch *stoch, int loaddata);
OSLLIBAPI int OSLLINKAGE  ekks_describeSubModel(EKKStoch *stoch,
													 int imodel,
													 int nnodes,
													 int *usrndlist,
													 int recalcprob);
  /* Copy stoch model (say, before you do something drastic to it) */
OSLLIBAPI int OSLLINKAGE  ekks_copyStoch(EKKStoch *target, EKKStoch *src);


  /*** c) EKKStoch methods to build scenario trees */

  /* Initialize EKKStoch structure for a maximum of mxsmp scenarios
     and create first event from the array of data values 
	 arr[] with narr entries.  The length of the arr[] array, narr,
	 is the maximum number of branch nodes between root and leaf. */
OSLLIBAPI int OSLLINKAGE ekks_createScenarioTree(EKKStoch *stoch,
								int *arr, int narr, int mxsmp, double dprob);
  /* Add event to EKKStoch object. Finds the event anc_arr[] with longest 
     sequence max{k: arr[0] = anc_arr[0], ... , arr[k] = anc_arr[k]},
	 and adds the branch (arr[k+1] ... arr[narr-1]).
     Returns < 0 if there is an error. */
OSLLIBAPI int OSLLINKAGE ekks_addEventToTree(EKKStoch *stoch, 
								int	*arr, int narr, double dprob);

  /* Get the scenario branching information for event number nsmp. 
     The meaning of the returned data is the same as in ekks_addScenario()*/
OSLLIBAPI int OSLLINKAGE ekks_getScenarioFromTree(EKKStoch *stoch,
		int nsmp, 
		int *ianc, int *istg, double *dprob, int *nscn,  
		int **arr);						  /* points to array values */

  /* Single function versions of ekks_getScenarioFromTree() */
OSLLIBAPI int OSLLINKAGE ekks_getScenarioAncestorFromTree(EKKStoch *stoch,int nsmp);
OSLLIBAPI int OSLLINKAGE ekks_getScenarioBranchStageFromTree(EKKStoch *stoch,int nsmp);
OSLLIBAPI double OSLLINKAGE ekks_getScenarioWeightFromTree(EKKStoch *stoch,int nsmp);
OSLLIBAPI int OSLLINKAGE ekks_getScenarioNumberFromTree(EKKStoch *stoch,int nsmp);
OSLLIBAPI int OSLLINKAGE ekks_getDataLengthFromTree(EKKStoch *stoch);
OSLLIBAPI int* OSLLINKAGE ekks_getScenarioDataFromTree(EKKStoch	*stoch,int nsmp);

  /* delete EKKStoch */
OSLLIBAPI int OSLLINKAGE  ekks_deleteTree(EKKStoch *stoch);
 
  /*****  
  7) Some other decomposition methods and ways to tune for them.  */

  /* The Nested L-shaped Method solver decomposes using the
     specified cutstrategy:										  */
#define CUTATSTAGE		1	/* equivalent to ekks_bendersLSolve   */
#define CUTBYROWSIZE	2	/* sets max # rows in subproblems     */
#define CUTBYMAXNODES	3	/* sets max # of subproblems          */

OSLLIBAPI int OSLLINKAGE  ekkse_nestedLSolve(EKKStoch *stoch, 
					int cutstrategy, int startmode);

  /* Get/set cut period for cut-at-stage decomposition */
OSLLIBAPI int OSLLINKAGE  ekks_getCutPeriod(EKKStoch *stoch);
OSLLIBAPI void OSLLINKAGE  ekks_setCutPeriod(EKKStoch *stoch, int period);

  /* Get/set number of rows for cut-by-row-size decomposition */
OSLLIBAPI int OSLLINKAGE ekks_getMinNumRows(EKKStoch *stoch);
OSLLIBAPI void OSLLINKAGE ekks_setMinNumRows(EKKStoch *stoch, int numrows);

  /* Get/set number of models for cut-by-max-models decomposition */
OSLLIBAPI int OSLLINKAGE ekks_getMaxsubmodels(EKKStoch *stoch);
OSLLIBAPI void OSLLINKAGE ekks_setMaxsubmodels(EKKStoch *stoch, int maxmodels);

  /* set direction of optimization */
OSLLIBAPI void OSLLINKAGE ekks_setMinimize(EKKStoch *stoch);
OSLLIBAPI void OSLLINKAGE ekks_setMaximize(EKKStoch *stoch);

  /* Get/set maximum number of major iterations in decomposition */
OSLLIBAPI int OSLLINKAGE ekks_getMaxiter(EKKStoch *stoch);
OSLLIBAPI void OSLLINKAGE  ekks_setMaxiter(EKKStoch *stoch, int maxiter);

  /* Get/set gap used to assess convergence of decomposition. */ 
OSLLIBAPI double OSLLINKAGE	ekks_getRelOptimalityGap(EKKStoch *stoch);
OSLLIBAPI void OSLLINKAGE  ekks_setRelOptimalityGap(EKKStoch *stoch, double tol);

  /* Set scaling on/off for subproblems in decomposition.
     It is usually a good idea to set it on. */
OSLLIBAPI void OSLLINKAGE ekks_setScaleOn(EKKStoch *stoch);
OSLLIBAPI void OSLLINKAGE ekks_setScaleOff(EKKStoch *stoch);
  
  /* Set OSL presolve type on/off for subproblems. */
OSLLIBAPI void OSLLINKAGE ekks_setPresolve(EKKStoch *stoch,int presolvetype);
OSLLIBAPI void OSLLINKAGE ekks_setPresolveOff(EKKStoch *stoch);

  /* Set OSL crash type on/off for subproblems */
OSLLIBAPI void OSLLINKAGE ekks_setCrash(EKKStoch *stoch, int crashmode);
OSLLIBAPI void OSLLINKAGE ekks_setCrashOff(EKKStoch *stoch);

  /* Set OSL Simplex solver type for subproblems. */
OSLLIBAPI void OSLLINKAGE ekks_setSimplexAlg(EKKStoch *stoch, int alg);

  /* After decomposition solve/don't solve with simplex solver */
OSLLIBAPI void OSLLINKAGE  ekks_setFinalSimplexSolverOn(EKKStoch *stoch);
OSLLIBAPI void OSLLINKAGE  ekks_setFinalSimplexSolverOff(EKKStoch *stoch);

  /* Set large bounds on/off (may affect speed of decomposition). */
OSLLIBAPI void OSLLINKAGE ekks_setLargeBoundsOn(EKKStoch *stoch);
OSLLIBAPI void OSLLINKAGE ekks_setLargeBoundsOff(EKKStoch *stoch);

  /* solve using OSL's simplex method */
OSLLIBAPI int OSLLINKAGE  ekks_eitherSimplex(EKKStoch *stoch);


  /*****
  8) Message handling, printing and accessing data */

    /* OSLSE Messages on/off */
OSLLIBAPI int OSLLINKAGE  ekks_messagePrintOff(EKKStoch *stoch,
	int extmsgnum);
OSLLIBAPI int OSLLINKAGE  ekks_messagePrintOn(EKKStoch *stoch,
		int extmsgnum);
   /* Lots of detail ... */
OSLLIBAPI void OSLLINKAGE  ekks_setDebugOn(EKKStoch *stoch);

  /* load model (to access data before solving) */
OSLLIBAPI int OSLLINKAGE ekks_loadBaseModel(EKKStoch *stoch);


OSLLIBAPI EKKModel * OSLLINKAGE  ekks_getCore(EKKContext *env,
											  EKKStoch *stoch);


  /* Stochastic LP data */
OSLLIBAPI int OSLLINKAGE ekks_estimateLPSize(EKKStoch *stoch);
OSLLIBAPI int OSLLINKAGE  ekks_getNumScenarios(EKKStoch *stoch);
OSLLIBAPI int OSLLINKAGE  ekks_getNumStages(EKKStoch *stoch);
OSLLIBAPI int OSLLINKAGE  ekks_getNumNodes(EKKStoch *stoch);
OSLLIBAPI int OSLLINKAGE  ekks_getCoreNumcols(EKKStoch *stoch);
OSLLIBAPI int OSLLINKAGE  ekks_getCoreNumrows(EKKStoch *stoch);

OSLLIBAPI int OSLLINKAGE  ekks_getNumcols(EKKStoch *stoch);
OSLLIBAPI int OSLLINKAGE  ekks_getNumrows(EKKStoch *stoch);

OSLLIBAPI int OSLLINKAGE  ekks_getNumcolsAtStage(EKKStoch *stoch,int stg);
OSLLIBAPI int OSLLINKAGE  ekks_getNumrowsAtStage(EKKStoch *stoch,int stg);


  /* Generate SMPS files --- useful for debugging! */
OSLLIBAPI int OSLLINKAGE  ekks_outMatrixSMPS(EKKStoch *stoch, 
	int smpstype, /* SCENARIOS, BLOCK_DISCRETE, INDEP_DISCRETE */ 
	int	replace,  /* ADD_TO_CORE_VALUES, REPLACE_CORE_VALUES */
	const char *corefile, const char *timefile, const char *stochfile);
  /* Generate MPS file */
OSLLIBAPI int OSLLINKAGE  ekks_outMatrixMPS(EKKStoch *stoch, const
		char *mpsfile);

  /* Solution Status and Obj Value
    (0 - optimal, 1- infeasible, 2 -unbounded, >=3 - incomplete) */
OSLLIBAPI int OSLLINKAGE  ekks_getStatus(EKKStoch *stoch);
OSLLIBAPI double OSLLINKAGE  ekks_getRobjvalue(EKKStoch *stoch);
  /* get number of times root subproblem was solved */
OSLLIBAPI int OSLLINKAGE  ekks_getNumberRootCycles(EKKStoch *stoch);
 
  /* Getting and Printing Solutions
      	printmode = 0 get column solution
		printmode = 1 get row solution
		
     Getting...
		OSLSE sorts the matrix into stage order.  The array
		index[] shows the OSL index number. */
OSLLIBAPI int OSLLINKAGE  ekks_getNodeSolution(EKKStoch *stoch,
	int scenario, int stage, int printmode, double *solution,
	int *index);
OSLLIBAPI int OSLLINKAGE  ekks_getScenarioSolution(EKKStoch *stoch,
	int scenario, int printmode, double *solution, int *index);
OSLLIBAPI int OSLLINKAGE  ekks_getNodeDualSolution(EKKStoch *stoch,
	int scenario, int stage, int printmode, double *solution,
	int *index);
OSLLIBAPI int OSLLINKAGE  ekks_getScenarioDualSolution(EKKStoch *stoch,
	int scenario, int printmode, double *solution, int *index);
OSLLIBAPI double OSLLINKAGE  ekks_getObjectiveDistributionExpectedValue(EKKStoch
	*stoch);

  /* Printing */
OSLLIBAPI int OSLLINKAGE  ekks_printObjectiveDistribution(EKKStoch
	*stoch);
OSLLIBAPI int OSLLINKAGE  ekks_printNodeSolution(EKKStoch *stoch,
	int scenario, int stage, int printmode);
OSLLIBAPI int OSLLINKAGE  ekks_printNodeDualSolution(EKKStoch *stoch,
	int scenario, int stage, int printmode);

 
  /******
  9) Stochastic Mixed Integer Problems								*/

OSLLIBAPI int OSLLINKAGE  ekks_setIntegersAtCore(EKKStoch *stoch,
	int numints, int *intnums, int *intType);
 /* ****
  ekks_setIntegersAtCore marks a set of columns in the core model as integers.
  The column numbers for the desired set are supplied by the user in an
  integer array, intnums[numints].  The integer array intType[numints]
  specifies the types of each integer 
  (1- binary, 2-general integer, 3-SOS2, 0- continuous).
 **** */

  /* propagate integer variables through EKKStoch and return total */
OSLLIBAPI int OSLLINKAGE  ekks_markIntegers(EKKStoch *stoch);
  /* propagate with priorities and pseudocosts */
OSLLIBAPI int OSLLINKAGE  ekks_markIntegersWithStagePriorities(EKKStoch *stoch, 
			int *stagePriority, double *upperPseudoCost, double *lowerPseudoCost);
  /* solve with branch and bound */ 
OSLLIBAPI int OSLLINKAGE  ekks_branchAndBound(EKKStoch *stoch,
		const char *matrixFile, const char *basisFile);


  /***
   Parametric solvers */

OSLLIBAPI int OSLLINKAGE  ekkse_solveParametricModel(EKKStoch *stoch,
		const double *parobj, const double *parrowlower, 
		const double *parrowupper, const double *parcollower, 
		const double * parcolupper, int mask, int (*report) (EKKModel *));
 /* ****
  ekkse_solveParametricModel solves the parametric full (deterministic)
  model. If the real control variable Rslambda is non-zero, then Rslambda
  times the perturbation vector is added to the original and the
  resulting problem is solved with the simplex method. Thereafter, the
  perturbation is added in increments multiplied by Rslambdadelta up to
  the maximum Rslambdalim.  
  It is more efficient to set up the problem so that the original is
  solved with OSLSE ekkse_nestedLSolve(), then call ekks_eitherSimplex()
  and ekks_solveParametricModel() with lambda=0.
  Each time the incremented problem is solved, the function pointed to
  by report() is called with the OSL EKKModel attached to stoch as argument.
  The mask argument is used as:
    (mask > 15) -- perform parametric analysis on the Column Upper Bounds Vector
    (mask > 7)  -- perform parametric analysis on the Column Lower Bounds Vector
    (mask > 3)  -- perform parametric analysis on the Row Upper Bounds Vector
    (mask > 1)  -- perform parametric analysis on the Row Lower Bounds Vector
    (mask > 0)  -- perform parametric analysis on the Objective Function Vector
 **** */

OSLLIBAPI int OSLLINKAGE  ekks_solveParametricCoreModel(EKKStoch *stoch,
		const double *parobj, const double *parrowlower, 
		const double *parrowupper, const double *parcollower, 
		const double * parcolupper, int mask, int (*report) (EKKModel *));

/* ***
   ekkse_solveParametricCoreModel first generates the parametric vectors
   for the full (deterministic equivalent) model.
   Each parametric vector input is assumed to be of the size of the core model.
   These vectors are sorted into time stages and then replicated as appropriate
   node by node to form the parametric vectors for the full model.
*/


  /***
   Examine decomposed subproblems. */
OSLLIBAPI int OSLLINKAGE ekkse_decomposeCutAtStage(EKKStoch *stoch, 
							int cutPeriod, int loadData);
OSLLIBAPI int OSLLINKAGE  ekkse_decomposeCutByRowSize(EKKStoch	*stoch,
							int minNumRows, int loadData);
OSLLIBAPI int OSLLINKAGE  ekkse_decomposeCutByNodeSize(EKKStoch *stoch, 
							int maxNumNodes, int loadData);
OSLLIBAPI int OSLLINKAGE  ekks_getNsubmodels(EKKStoch *stoch);
OSLLIBAPI EKKStoch ** OSLLINKAGE  ekks_getSubmodels(EKKStoch *stoch);


  /* access to OSL version of EKKStoch model */
OSLLIBAPI int OSLLINKAGE  ekkse_copyCurrentModel(EKKStoch *stoch,
	EKKModel *model);
OSLLIBAPI EKKModel * OSLLINKAGE  ekkse_getBaseModel(EKKStoch *stoch);
OSLLIBAPI EKKModel * OSLLINKAGE  ekkse_getCurrentModel(EKKStoch *stoch);


  /*** Parallel */
OSLLIBAPI int OSLLINKAGE  ekkse_setParallelOn(EKKStoch *stoch);
OSLLIBAPI void OSLLINKAGE  ekkse_setParallelOnAfterVM(EKKStoch *stoch);
OSLLIBAPI void OSLLINKAGE  ekkse_setParallelOff(EKKStoch *stoch);

/* these are obsolete */
OSLLIBAPI int OSLLINKAGE  ekks_initialize();
OSLLIBAPI int OSLLINKAGE  ekks_end();
OSLLIBAPI EKKStoch * OSLLINKAGE  ekks_createStoch(const char *modelname, int maxscn);
#endif
#ifdef __cplusplus
 }
#endif
